from haystack import indexes

from django.db.models import signals

from tendenci.core.perms.object_perms import ObjectPermission
from tendenci.apps.search.indexes import CustomSearchIndex


class TendenciBaseSearchIndex(CustomSearchIndex):
    text = indexes.CharField(document=True, use_template=True)
    
    # TendenciBaseModel Fields
    allow_anonymous_view = indexes.BooleanField(model_attr='allow_anonymous_view')
    allow_user_view = indexes.BooleanField(model_attr='allow_user_view')
    allow_member_view = indexes.BooleanField(model_attr='allow_member_view')
    # allow_anonymous_edit = indexes.BooleanField(model_attr='allow_anonymous_edit')
    allow_user_edit = indexes.BooleanField(model_attr='allow_user_edit')
    allow_member_edit = indexes.BooleanField(model_attr='allow_member_edit')
    creator = indexes.CharField(model_attr='creator', null=True)
    creator_username = indexes.CharField(model_attr='creator_username', null=True)
    owner = indexes.CharField(model_attr='owner', null=True)
    owner_username = indexes.CharField(model_attr='owner_username', null=True)
    status = indexes.BooleanField(model_attr='status')
    status_detail = indexes.CharField(model_attr='status_detail')
    create_dt = indexes.DateTimeField(model_attr='create_dt', null=True)
    update_dt = indexes.DateTimeField(model_attr='update_dt', null=True)

    # permission fields
    users_can_view = indexes.MultiValueField(null=True)
    groups_can_view = indexes.MultiValueField(null=True)
    
    # PK: needed for exclude list_tags
    primary_key = indexes.CharField(model_attr='pk')

    def get_updated_field(self):
        return 'update_dt'

    def prepare_users_can_view(self, obj):
        """
        This needs to be overwritten if 'view' permission label does not follow the standard convention:
        (app_label).view_(module_name)
        """
        return ObjectPermission.objects.users_with_perms('%s.view_%s' % (obj._meta.app_label, obj._meta.module_name), obj)

    def prepare_groups_can_view(self, obj):
        """
        This needs to be overwritten if 'view' permission label does not follow the standard convention:
        (app_label).view_(module_name)
        """
        return ObjectPermission.objects.groups_with_perms('%s.view_%s' % (obj._meta.app_label, obj._meta.module_name), obj)




from geraldo import Report, ReportBand, ObjectValue, SystemField,\
    BAND_WIDTH, Label, landscape
from reportlab.lib.units import cm
from reportlab.lib.enums import TA_RIGHT, TA_CENTER
from reportlab.lib.pagesizes import A5
from datetime import datetime, timedelta

class ReportBandNewMems(ReportBand):
    def __init__(self, *args, **kwargs):
        days_ago = kwargs.pop('days_ago')
        super(ReportBandNewsMems, self).__init__(*args, **kwargs)

class ReportNewMems(Report):
    title = "New Memberships"
    author = "John Smith  Corporation"
    
    page_size = landscape(A5)
    
    def __init__(self, *args, **kwargs):
        super(ReportNewMems, self).__init__(*args, **kwargs)
    
    class band_page_header(ReportBand):
        height = 1.2*cm
        elements = [
                Label(text="Name", top=0.8*cm, left=0*cm),
                Label(text="Email", top=0.8*cm, left=2.5*cm),
                Label(text="Type", top=0.8*cm, left=5.5*cm),
                Label(text="Application", top=0.8*cm, left=8.5*cm),
                Label(text="Price Paid", top=0.8*cm, left=11.5*cm),
                Label(text="Start Date", top=0.8*cm, left=14.5*cm),
                Label(text="End Date", top=0.8*cm, left=17.5*cm),
            ]
        
    class band_detail(ReportBand):
        height = 0.5*cm
        elements = (
                ObjectValue(attribute_name='user', left=0*cm,
                    get_value=lambda instance: instance.user.last_name + ', ' + instance.user.first_name),
                ObjectValue(attribute_name='user', left=2.5*cm, 
                    get_value=lambda instance: instance.user.email),
                ObjectValue(attribute_name='membership_type', left=5.5*cm),
                ObjectValue(attribute_name='ma', left=8.5*cm),
                ObjectValue(attribute_name='invoice', left=11.5*cm,
                    get_value=lambda instance: instance.invoice.total if instance.invoice else '' ),
                #ObjectValue(attribute_name='payment_method', left=15*cm),
                ObjectValue(attribute_name='subscribe_dt', left=14.5*cm,
                    get_value=lambda instance: instance.subscribe_dt.strftime('%m/%d/%Y')),
                ObjectValue(attribute_name='expire_dt', left=17.5*cm,
                    get_value=lambda instance: instance.expire_dt.strftime('%m/%d/%Y') if instance.expire_dt else ''),
            )
    



from datetime import datetime
import operator
from django.db.models import Q

from django.contrib.auth.models import User, AnonymousUser

from tendenci.core.site_settings.utils import get_setting
from tendenci.addons.events.models import Event, Addon, AddonOption

def get_active_addons(event):
    """
    Returns all active addons of an event.
    """
    addons = Addon.objects.filter(event=event, status=True)
    return addons

def get_available_addons(event, user, is_strict=False):
    """
    Get the available addons of an event for user.
    """
    filter_and, filter_or = get_addon_access_filter(user, is_strict=is_strict)
    q_obj = None
    if filter_and:
        q_obj = Q(**filter_and)
    if filter_or:
        q_obj_or = reduce(operator.or_, [Q(**{key: value}) for key, value in filter_or.items()])
        if q_obj:
            q_obj = reduce(operator.and_, [q_obj, q_obj_or])
        else:
            q_obj = q_obj_or
            
    addons = Addon.objects.filter(
                event=event,
                status=True
                )
    if q_obj:
        addons = addons.filter(q_obj)
        
    return addons

def get_addon_access_filter(user, is_strict=False):
    if user.profile.is_superuser: return None, None
    
    filter_and, filter_or = None, None
    
    if is_strict:
        if user.is_anonymous():
            filter_or = {'allow_anonymous': True}
        elif not user.profile.is_member:
            filter_or = {'allow_anonymous': True,
                         'allow_user': True
                        }
        else:
            # user is a member
            filter_or = {'allow_anonymous': True,
                         'allow_user': True,
                         'allow_member': True}
            # get a list of groups for this user
            groups_id_list = user.group_member.values_list('group__id', flat=True)
            if groups_id_list:
                filter_or.update({'group__id__in': groups_id_list})
    else:
        filter_or = {'allow_anonymous': True,
                    'allow_user': True,
                    'allow_member': True,
                    'group__id__gt': 0}

            
    return filter_and, filter_or
    
    
    
def get_addons_for_list(event, users):
    """
    Returns the available addons of an event for a given list of users.
    """
    addons = Addon.objects.none()
    
    for user in users:
        addons = addons | get_available_addons(event, user)
    addons = addons | get_available_addons(event, AnonymousUser())
    
    # return the QUERYSET
    return addons
    
def can_use_addon(event, user, addon):
    """
    Determine if a user can use a specific addon of a given event
    """
    addons = get_available_addons(event, user)
    return addons.filter(pk=addon.pk).exists()
    
def create_regaddon(form, event, reg8n):
    """
    Create the RegAddon.
    form is a RegAddonForm with the specifed addon options.
    reg8n is the Registration instance to associate the regaddon with.
    """
    
    regaddon = RegAddon()
    
    return regaddon
    



from tendenci.core.base.test import TestCase
from selenium.common.exceptions import NoSuchElementException


class MembershipTest(TestCase):

    def test_create_membership_entry(self):
        from tendenci.addons.memberships.models import App
        apps = App.objects.all()

        if not apps:
            return None

        self.browser.get('http://127.0.0.1:8000/memberships/%s/' % apps[0].slug)
        form_fields = self.browser.find_elements_by_css_selector('.form-builder-wrap .form-field')

        for form_field in form_fields:

            try:
                label = form_field.find_element_by_css_selector('label')
                input_field = form_field.find_element_by_css_selector('input')
            except NoSuchElementException:
                continue  # on to the next one

            try:
                is_required = form_field.find_element_by_css_selector('div.label.required').get_attribute('class')
            except NoSuchElementException:
                is_required = False

            label = label.text
            if input_field.is_displayed():
                attr_type = input_field.get_attribute('type')

                if attr_type == 'text' and is_required:
                    if 'first name' in label.lower():
                        input_field.send_keys(self.user.first_name)
                    elif 'last name' in label.lower():
                        input_field.send_keys(self.user.last_name)
                    elif 'email' in label.lower():
                        input_field.send_keys(self.user.email)
                    else:
                        input_field.send_keys('blah')
                elif attr_type in ['checkbox', 'radio']:
                    input_field.click()

        self.save_screenshot()
        submit_btn = self.browser.find_element_by_css_selector('.form-builder-wrap input[type="submit"]')
        submit_btn.click()
        self.save_screenshot()



import os
import re
from datetime import datetime
from haystack.query import SearchQuerySet

from django.contrib.contenttypes.models import ContentType
from django.db.models import Manager
from django.conf import settings
from django.core.files.storage import default_storage

from tendenci.core.perms.managers import TendenciBaseManager


def save_to_disk(f, instance):
    """
    Takes file object and instance (or model).
    Returns back relative path of file.
    """

    file_name = re.sub(r'[^a-zA-Z0-9._]+', '-', f.name)

    # make dir with app and module name
    relative_directory = os.path.join(
        'files',
        instance._meta.app_label,
        instance._meta.module_name,
    )

    # make directory with pk
    if isinstance(instance.pk, long):
        relative_directory = os.path.join(
            relative_directory,
            unicode(instance.pk),
        )

    default_storage.save(os.path.join(relative_directory, file_name), f)

#    absolute_directory = os.path.join(settings.MEDIA_ROOT, relative_directory)
#
#    if not os.path.exists(absolute_directory):
#        os.makedirs(absolute_directory)
#
#    destination = open(os.path.join(absolute_directory, file_name), 'wb+')
#    for chunk in f.chunks():
#        destination.write(chunk)
#    destination.close()

    # relative path
    return os.path.join(relative_directory, file_name)

class FileManager(TendenciBaseManager):
    """
    Model Manager
    """

    def get_for_model(self, instance):
        return self.model.objects.filter(
            content_type=ContentType.objects.get_for_model(instance),
            object_id=instance.pk,
        )

    def bind_files_to_instance(self, files, instance, **kwargs):
        """
        Save files and associate with instance.
        Return list of files saved.
        """
        from django.contrib.auth.models import User

        try:  # explicit user; default to admin
            user = kwargs.get('user') or User.objects.get(id=1)
        except User.DoesNotExist as e:
            return []

        # loop; save file; save file record in db
        # ----------------------------------------

        files_saved = []
        for file in files:

            # what to save; where to save it
            file.file.seek(0)
            file_path = save_to_disk(file, instance)

            # update file record; or create new file record
            # ----------------------------------------------

            instance_pk = None
            if isinstance(instance.pk, long):
                instance_pk = instance.pk

            try:
                file = self.get(file=file_path)
                file.name = file.name
                file.owner = user
                file.owner_username = user.username
                file.update_dt = datetime.now()
            except:
                file = self.model(**{
                    'file':file_path,
                    'name':file.name,
                    'content_type':ContentType.objects.get_for_model(instance),
                    'object_id':instance_pk,
                    'creator':user,
                    'creator_username':user.username,
                    'owner':user,
                    'owner_username':user.username,
                })

            file.save() # auto generate GUID if missing
            files_saved.append(file)

        return files_saved

    def save_files_for_instance(self, request, instance, **kwargs):
        """
        Save files and associate with instance.
        Return list of files saved.
        """

        files = request.FILES.values()
        if 'files' in kwargs:  # 'files' key overwrites request
            files = kwargs.get('files') or []

        # loop; save file; save file record in db
        # ----------------------------------------
        files_saved = []
        for file in files:

            file_path = save_to_disk(file, instance)

            # update file record; or create new file record
            # ----------------------------------------------
            instance_pk = None
            if isinstance(instance.pk, long) or isinstance(instance.pk, int):
                instance_pk = instance.pk

            try:
                file = self.get(file=file_path)
                file.name = file.name
                file.owner = request.user
                file.owner_username = request.user.username
                file.update_dt = datetime.now()
                file.content_type = ContentType.objects.get_for_model(instance)
                file.object_id = instance_pk
            except:
                file = self.model(**{
                    'file': file_path,
                    'name': file.name,
                    'content_type': ContentType.objects.get_for_model(instance),
                    'object_id': instance_pk,
                    'creator': request.user,
                    'creator_username': request.user.username,
                    'owner': request.user,
                    'owner_username': request.user.username,
                })

            file.save() # auto generate GUID if missing
            files_saved.append(file)

        return files_saved


from datetime import datetime
from django.template import Library, TemplateSyntaxError, Variable
from tendenci.core.base.template_tags import ListNode, parse_tag_kwargs
from tendenci.core.perms.utils import get_query_filters
from django.contrib.auth.models import AnonymousUser, User
from tendenci.apps.navs.models import Nav
from tendenci.apps.navs.utils import get_nav, cache_nav

register = Library()

@register.inclusion_tag("navs/options.html", takes_context=True)
def nav_options(context, user, nav):
    context.update({
        "opt_object": nav,
        "user": user
    })
    return context

@register.inclusion_tag("navs/nav.html", takes_context=True)
def nav_nav(context, user, nav=None):
    context.update({
        "nav_object": nav,
        "user": user
    })
    return context

@register.inclusion_tag("navs/search-form.html", takes_context=True)
def nav_search(context):
    return context
    
@register.inclusion_tag("navs/navigation.html", takes_context=True)
def navigation(context, nav_id):
    """
    Renders the nav and its nav items.
    This will call nav_item that will call itself recursively nesting 
    the subnavs
    """
    user = AnonymousUser()
    
    if 'user' in context:
        if isinstance(context['user'], User):
            user = context['user']
    try:
        filters = get_query_filters(user, 'navs.view_nav')
        navs = Nav.objects.filter(filters).filter(id=nav_id)
        if user.is_authenticated():
            if not user.profile.is_superuser:
                navs = navs.distinct()
        nav = navs[0]
    except:
        return None
    context.update({
        "nav": nav,
        "items": nav.top_items,
    })
    return context

@register.inclusion_tag("navs/load_nav.html", takes_context=True)
def load_nav(context, nav_id):
    """
    Renders the nav and its nav items.
    This will call nav_item that will call itself recursively nesting 
    the subnavs
    """
    # No perms check because load_nav is only called by the other tags
    try:
        nav = Nav.objects.get(id=nav_id)
    except:
        return None
    context.update({
        "nav": nav,
        "items": nav.top_items,
    })
    return context

@register.inclusion_tag("navs/navigation_item.html", takes_context=True)
def nav_item(context, item):
    """
        Renders a nav item and its children.
    """
    context.update({
        "item": item,
    })
    return context

@register.inclusion_tag("navs/cached_nav.html", takes_context=True)
def nav(context, nav_id):
    """
    Renders the nav from cache
    if not will use the navigation tag for rendering the nav
    """
    user = AnonymousUser()

    if 'user' in context:
        if isinstance(context['user'], User):
            user = context['user']
    try:
        filters = get_query_filters(user, 'navs.view_nav')
        navs = Nav.objects.filter(filters).filter(id=nav_id)
        if user.is_authenticated():
            if not user.profile.is_superuser:
                navs = navs.distinct()

        nav_object = navs[0]
        nav = get_nav(nav_object.pk)
        if not nav:
            cache_nav(nav_object)
    except:
        return None

    context.update({
        "cached": nav,
        "nav_id": nav_id,
    })
    return context


class ListNavNode(ListNode):
    model = Nav
    perms = 'navs.view_nav'


@register.tag
def list_navs(parser, token):
    """
    Used to pull a list of :model:`navs.Nav` items.

    Usage::

        {% list_case_studies as [varname] [options] %}

    Be sure the [varname] has a specific name like ``case_studies_sidebar`` or 
    ``case_studies_list``. Options can be used as [option]=[value]. Wrap text values
    in quotes like ``tags="cool"``. Options include:
    
        ``limit``
           The number of items that are shown. **Default: 3**
        ``order``
           The order of the items. **Default: ID**
        ``user``
           Specify a user to only show public items to all. **Default: Viewing user**
        ``query``
           The text to search for items. Will not affect order.
        ``tags``
           The tags required on items to be included.
        ``random``
           Use this with a value of true to randomize the items included.

    Example::

        {% list_navs as nav_list limit=5 tags="cool" %}
        {% for nav in nav_list %}
            {% nav nav %}
        {% endfor %}
    """
    args, kwargs = [], {}
    bits = token.split_contents()
    context_var = bits[2]

    if len(bits) < 3:
        message = "'%s' tag requires at least 2 parameters" % bits[0]
        raise TemplateSyntaxError(message)

    if bits[1] != "as":
        message = "'%s' second argument must be 'as'" % bits[0]
        raise TemplateSyntaxError(message)

    kwargs = parse_tag_kwargs(bits)

    if 'order' not in kwargs:
        kwargs['order'] = 'pk'

    return ListNavNode(context_var, *args, **kwargs)



from datetime import datetime
from datetime import timedelta
from decimal import Decimal

from django.utils.translation import ugettext_lazy as _
from django import forms
from tendenci.core.perms.forms import TendenciBaseForm
from tendenci.apps.discounts.models import Discount
from tendenci.core.base.fields import SplitDateTimeField
from tendenci.apps.discounts.utils import assign_discount

END_DT_INITIAL = datetime.now() + timedelta(weeks=4)

class DiscountForm(TendenciBaseForm):
    class Meta:
        model = Discount
        fields = (
            'discount_code',
            'value',
            'start_dt',
            'end_dt',
            'never_expires',
            'cap',
            'allow_anonymous_view',
            'user_perms',
            'group_perms',
            'status',
            'status_detail',
            )

        fieldsets = [('Discount Information', {
                      'fields': ['discount_code',
                                 'value',
                                 'cap',
                                 'never_expires',
                                 'start_dt',
                                 'end_dt',
                                 ],
                      'legend': ''
                      }),
                      ('Permissions', {
                      'fields': ['allow_anonymous_view',
                                 'user_perms',
                                 'member_perms',
                                 'group_perms',
                                 ],
                      'classes': ['permissions'],
                      }),
                     ('Administrator Only', {
                      'fields': ['status',
                                 'status_detail'],
                      'classes': ['admin-only'],
                    })
                    ]
        
    start_dt = SplitDateTimeField(label=_('Start Date/Time'), initial=datetime.now())
    end_dt = SplitDateTimeField(label=_('End Date/Time'), initial=END_DT_INITIAL)
    status_detail = forms.ChoiceField(
        choices=(('active','Active'),('inactive','Inactive'), ('pending','Pending'),))
        
    def __init__(self, *args, **kwargs):
        super(DiscountForm, self).__init__(*args, **kwargs)
        if not self.user.profile.is_superuser:
            if 'status' in self.fields: self.fields.pop('status')
            if 'status_detail' in self.fields: self.fields.pop('status_detail')
            
    def clean_discount_code(self):
        data = self.cleaned_data['discount_code']
        try:
            discount = Discount.objects.get(discount_code=data)
        except Discount.DoesNotExist:
            return data
        if not discount == self.instance:
            raise forms.ValidationError('There a discount for this code already exists.')
        return data

    def clean(self):
        cleaned_data = self.cleaned_data
        start_dt = cleaned_data.get("start_dt")
        end_dt = cleaned_data.get("end_dt")

        if start_dt > end_dt:
            errors = self._errors.setdefault("end_dt", ErrorList())
            errors.append(u"This cannot be \
                earlier than the start date.")

        # Always return the full collection of cleaned data.
        return cleaned_data

class DiscountCodeForm(forms.Form):
    price = forms.DecimalField(decimal_places=2)
    code = forms.CharField()
    count = forms.IntegerField()
    
    def clean(self):
        code = self.cleaned_data.get('code', '')
        count = self.cleaned_data.get('count', 1)
        try:
            discount = Discount.objects.get(discount_code=code)
        except Discount.DoesNotExist:
            raise forms.ValidationError('This is not a valid discount code.')
        if not discount.available_for(count):
            raise forms.ValidationError('This is not a valid discount code.')
        return self.cleaned_data
        
    def new_price(self):
        code = self.cleaned_data['code']
        price = self.cleaned_data['price']
        count = self.cleaned_data['count']
        discount = Discount.objects.get(discount_code=code).value * Decimal(count)
        new_price = price - discount
        if new_price < 0:
            new_price = Decimal('0.00')
        return (new_price, discount)
    
    
class DiscountHandlingForm(forms.Form):
    """
    Process a list of prices, and returns a list of discounted prices.
    """
    prices = forms.CharField()
    code = forms.CharField()
    
    def clean(self):
        code = self.cleaned_data.get('code', '')
        [self.discount] = Discount.objects.filter(discount_code=code)[:1] or [None]
        if not self.discount:
            raise forms.ValidationError('This is not a valid discount code.')
        
        if not self.discount.never_expires:
            now = datetime.now()
            if self.discount.start_dt > now:
                raise forms.ValidationError('This discount code is not in effect yet.')
            if self.discount.end_dt <= now:    
                raise forms.ValidationError('This discount code has expired.')
        
        self.limit = 0
        if self.discount.cap != 0:
            self.limit = self.discount.cap - self.discount.num_of_uses()
            if self.limit <= 0:
                raise forms.ValidationError('This discount code has passed the limit.')
        
        return self.cleaned_data
        
    def get_discounted_prices(self):
        prices = self.cleaned_data['prices']
        price_list = [Decimal(price) for price in prices.split(';')]
        
        return assign_discount(price_list, self.discount)
        
            
            
        
        
        




from tastypie.authorization import Authorization
from tastypie.resources import ModelResource
from tastypie import fields

from tendenci.core.api_tasty.auth import DeveloperApiKeyAuthentication
from tendenci.core.api_tasty.serializers import SafeSerializer
from tendenci.core.payments.models import PaymentMethod

class PaymentMethodResource(ModelResource):
    class Meta:
        queryset = PaymentMethod.objects.all()
        resource_name = 'payment_method'
        serializer = SafeSerializer()
        authorization = Authorization()
        authentication = DeveloperApiKeyAuthentication()
        list_allowed_methods = ['get',]
        detail_allowed_methods = ['get',]



from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = "Removes migrations for an app. Requires an app name to be passed"

    def handle(self, *args, **options):
        """Removes migrations for an app."""
        from south.models import MigrationHistory
        app_name = args[0]
        if app_name:
            migrations = MigrationHistory.objects.filter(app_name=app_name)
            if migrations:
                migrations.delete()
                print "Migrations for %s were removed" % app_name
            else:
                print "No migrations for %s" % app_name



from django.core.management.base import BaseCommand


class Command(BaseCommand):
    """
    Send recap email once event closes.

    An event registration is closed if the registration end date
    is less than current date.
    This command must be handled by asynchronous task queue/job
    """
    def handle(self, *args, **options):
        from tendenci.core.base.utils import send_email_notification
        from tendenci.core.site_settings.utils import get_setting
        from tendenci.addons.events.models import Event
        from tendenci.addons.events.utils import registration_has_recently_ended

        site_label = get_setting('site', 'global', 'sitedisplayname')
        site_url = get_setting('site', 'global', 'siteurl')
        admins = get_setting('module', 'events', 'admin_emails').split(',')
        email_list = [admin.strip() for admin in admins]

        # Query all events that are not yet marked for checking
        events = Event.objects.filter(mark_registration_ended=False)
        for event in events:
            if registration_has_recently_ended(event):
                # Calculate fees, number of participants, etc
                money_collected = event.money_collected
                money_outstanding = event.money_outstanding
                all_registrants = event.registrants()
                registrants_with_balance = event.registrants(with_balance=True)
                print 'Sending email to admins for event %s.' % (event, )
                send_email_notification(
                    'event_registration_end_recap',
                    email_list,
                    {
                        'SITE_GLOBAL_SITEDISPLAYNAME': site_label,
                        'SITE_GLOBAL_SITEURL': site_url,
                        'event': event,
                        'money_collected': money_collected,
                        'money_outstanding': money_outstanding,
                        'registrants_count': len(all_registrants),
                        'registrants_with_balance_count': len(registrants_with_balance),
                    }
                )
                print 'Message sent.'

                # Mark event as ended
                event.mark_registration_ended = True
                event.save()



from django.conf.urls.defaults import *


urlpatterns = patterns("tendenci.apps.subscribers.views",
    url(r"^(?P<id>\d+)/$", "subscribers", name="subscribers"),
    url(r"^detail/(?P<id>\d+)/$", "subscriber_detail", name="subscriber_detail"),
    url(r"^delete/(?P<id>\d+)/$", "subscriber_delete", name="subscriber_delete"),
)



from django.shortcuts import render_to_response
from django.http import Http404, HttpResponseRedirect
from django.template import RequestContext
from django.contrib import messages

from tendenci.core.base.http import Http403
from tendenci.core.site_settings.models import Setting
from tendenci.core.site_settings.forms import build_settings_form
from tendenci.core.site_settings.utils import delete_settings_cache
from tendenci.core.perms.utils import has_perm


def list(request, scope, scope_category, template_name="site_settings/list.html"):
    if not has_perm(request.user, 'site_settings.change_setting'):
        raise Http403

    settings = Setting.objects.filter(scope=scope, scope_category=scope_category).order_by('label')
    if not settings:
        raise Http404

    if request.method == 'POST':
        form = build_settings_form(request.user, settings)(request.POST, request.FILES)
        if form.is_valid():
            # this save method is overriden in the forms.py
            form.save()
            delete_settings_cache(scope, scope_category)
            try:
                if form.cleaned_data['theme']:
                    from django.core.management import call_command
                    call_command('hide_settings', 'theme')
                    call_command('update_settings', 'themes.%s' % form.cleaned_data['theme'].lstrip())
            except:
                pass

#             # if localizationlanguage is changed, update local settings
#             from django.conf import settings as django_settings
#             lang = get_setting('site', 'global', 'localizationlanguage')
#             #if lang in ['en-us', 'es']
#             if django_settings.LANGUAGE_CODE <> lang:
#                 local_setting_file = os.path.join(getattr(django_settings, 'PROJECT_ROOT'), 'local.py')
#                 f = open(local_setting_file, 'r')
#                 content = f.read()
#                 f.close()
# 
#                 if content.find('LANGUAGE_CODE') == -1:
#                     # we don't have LANGUAGE_CODE in local_settings, just append to it
#                     content = '%s\nLANGUAGE_CODE=\'%s\'\n' % (content, lang)
#                 else:
#                     p = re.compile(r'([\d\D\s\S\w\W]*?LANGUAGE_CODE\s*=\s*[\'\"])([\w-]+)([\'\"][\d\D\s\S\w\W]*?)')
#                     
#                     content = p.sub(r'\1%s\3' % lang, content)
# 
#                 f = open(local_setting_file, 'w')
#                 f.write(content)
#                 f.close()
# 
#                 from django.core.management import call_command
#                 call_command('touch_settings')
#                 #setattr(django_settings, 'LANGUAGE_CODE', lang)

            messages.add_message(request, messages.SUCCESS, 'Successfully saved %s settings' % scope_category.replace('_',' ').title())

            redirect_to = request.REQUEST.get('next', '')
            if redirect_to:
                return HttpResponseRedirect(redirect_to)

    else:
        form = build_settings_form(request.user, settings)()

    return render_to_response(template_name, {'form': form, 
                                              'scope_category': scope_category }, context_instance=RequestContext(request))


def index(request, template_name="site_settings/settings.html"):
    if not has_perm(request.user,'site_settings.change_setting'):
        raise Http403
    settings = Setting.objects.values().exclude(scope='template').order_by('scope_category')
    return render_to_response(template_name, {'settings':settings}, context_instance=RequestContext(request))


def single_setting(request, scope, scope_category, name, template_name="site_settings/list.html"):
    if not has_perm(request.user,'site_settings.change_setting'):
        raise Http403
    
    settings = Setting.objects.filter(scope=scope, scope_category=scope_category, name=name).order_by('label')
    if not settings:
        raise Http404
    
    if request.method == 'POST':
        form = build_settings_form(request.user, settings)(request.POST, request.FILES)
        if form.is_valid():
            # this save method is overriden in the forms.py
            form.save()
            try:
                if form.cleaned_data['theme']:
                    from django.core.management import call_command
                    call_command('hide_settings', 'theme')
                    call_command('update_settings', 'themes.%s' % form.cleaned_data['theme'].lstrip())
            except:
                pass

            messages.add_message(request, messages.SUCCESS, 'Successfully saved %s settings' % name.replace('_',' ').title())

            redirect_to = request.REQUEST.get('next', '')
            if redirect_to:
                return HttpResponseRedirect(redirect_to)

    else:
        form = build_settings_form(request.user, settings)()
        
    return render_to_response(template_name, {'form': form }, context_instance=RequestContext(request))




from datetime import datetime, timedelta
from django.core.management.base import BaseCommand
from django.template.loader import render_to_string


class Command(BaseCommand):
    """
    Send email reminders to the event registrants requesting reminders.

    Usage: ./manage.py send_event_reminders --verbosity=2
    """
    def send_reminders(self, event, registrants, verbosity=0):
        email = event.email

        count = 0
        for registrant in registrants:
            if registrant.email:
                if verbosity == '2':
                    print 'Sending reminder email to %s %s' % (
                                registrant.first_name,
                                registrant.last_name)
                email.recipient = registrant.email
                email.send()
                count += 1

        if count > 0:
            # notify event organizer that reminders have been
            # distributed to all registrants requesting reminders.
            self.send_organizer_confirmation(event)

    def get_reminder_conf_body(self, event):
        from tendenci.core.site_settings.utils import get_setting
        template_name = 'events/reminder_conf_email.html'

        return render_to_string(template_name,
                       {'event': event,
                        'site_url': get_setting('site', 'global',
                                                'siteurl'),
                        'site_display_name': get_setting(
                                                 'site',
                                                 'global',
                                                 'sitedisplayname')
                        })

    def send_organizer_confirmation(self, event):
        from tendenci.core.emails.models import Email
        from tendenci.core.site_settings.utils import get_setting

        email = Email()
        if event.organizer and event.organizer.user \
            and event.organizer.user.email:
            email.recipient = event.organizer.user.email
        else:
            email.recipient = get_setting('module', 'events', 'admin_emails')
            if not email.recipient:
                email.recipient = get_setting('site', 'global',
                                              'sitecontactemail')

        email.subject = '%s Event Reminders Distributed for: %s' % (
                                get_setting('site', 'global',
                                            'sitedisplayname'),
                                event.title
                                )
        email.body = self.get_reminder_conf_body(event)

        email.send()

    def handle(self, *args, **options):
        from tendenci.addons.events.models import Event, Registrant, Organizer
        from tendenci.core.emails.models import Email
        from tendenci.core.site_settings.utils import get_setting
        from tendenci.core.base.utils import convert_absolute_urls
        from tendenci.addons.events.utils import (render_event_email,
                                  get_default_reminder_template)

        verbosity = options['verbosity']
        site_url = get_setting('site', 'global', 'siteurl')
        now = datetime.now()
        today_tuple = (datetime(now.year, now.month, now.day, 0, 0, 0),
                       datetime(now.year, now.month, now.day, 23, 59, 59))

        # get a list of upcoming events that are specified to send reminders.
        events = Event.objects.filter(start_dt__gt=now,
                                registration_configuration__enabled=True,
                                registration_configuration__send_reminder=True,
                                status=True,
                                status_detail='active')
        events_list = []

        if events:
            for event in events:
                reg_conf = event.registration_configuration
                reminder_days = reg_conf.reminder_days
                if not reminder_days:
                    reminder_days = '1'
                days_list = reminder_days.split(',')

                for day in days_list:
                    try:
                        day = int(day)
                    except:
                        continue

                    start_dt = event.start_dt - timedelta(days=day)

                    if today_tuple[0] <= start_dt and start_dt <= today_tuple[1]:
                        events_list.append(event)

                        break

            for event in events_list:
                registrants = Registrant.objects.filter(
                                reminder=True,
                                registration__event=event
                                )

                reg_conf = event.registration_configuration
                [organizer] = Organizer.objects.filter(event=event)[:1] or [None]
                event.organizer = organizer

                email = reg_conf.email
                if not email:
                    email = Email()

                if not email.sender_display:
                    if organizer.name:
                        email.sender_display = organizer.name

                if not email.reply_to:
                    if organizer.user and organizer.user.email:
                        email.reply_to = organizer.user.email

                if not email.subject:
                    email.subject = 'Reminder: %s' % event.title
                else:
                    email.subject = 'Reminder: %s' % email.subject

                if not email.body:
                    email.body = get_default_reminder_template(event)

                email = render_event_email(event, email)

                # replace the relative links with absolute urls
                # in the email body and subject
                email.body = convert_absolute_urls(email.body, site_url)

                event.email = email
                self.send_reminders(event, registrants, verbosity=verbosity)



"""This middleware takes the session identifier in a POST message and adds it to the cookies instead.

This is necessary because SWFUpload won't send proper cookies back; instead, all the cookies are
added to the form that gets POST-ed back to us.
"""

from django.conf import settings
from django.core.urlresolvers import reverse

class SWFUploadMiddleware(object):
    def process_request(self, request):
        swf_cookie_name = settings.SWFUPLOAD_COOKIE_NAME
        if request.method == 'POST':
            if request.POST.has_key("photoset_id"):
                photoset_id = int(request.POST["photoset_id"])
                if (request.path == reverse('photos_batch_add', args=[photoset_id])) and \
                        request.POST.has_key(swf_cookie_name):
                    
                    request.COOKIES[settings.SESSION_COOKIE_NAME] = request.POST[swf_cookie_name]
                if request.POST.has_key('csrftoken'):           
                    request.COOKIES["csrftoken"] = request.POST['csrftoken']

class MediaUploadMiddleware(object):
    def process_request(self, request):
        swf_cookie_name = settings.SWFUPLOAD_COOKIE_NAME
        
        if (request.method == 'POST') and (request.path == reverse('file.swfupload')) and \
                request.POST.has_key(swf_cookie_name):
            request.COOKIES[settings.SESSION_COOKIE_NAME] = request.POST[swf_cookie_name]
            
        if request.POST.has_key('csrftoken'):           
            request.COOKIES["csrftoken"] = request.POST['csrftoken']
            
    def process_response(self, request, response):
        # set cookie for swfupload
        if request.COOKIES.has_key(settings.SESSION_COOKIE_NAME):
            response.set_cookie(settings.SWFUPLOAD_COOKIE_NAME,
                                request.COOKIES[settings.SESSION_COOKIE_NAME])
        return response



from django import forms
from django.forms import ModelForm
from django.conf import settings
from django.utils.translation import gettext as _
from tendenci.apps.wp_importer.models import BlogImport

class BlogImportForm(forms.ModelForm):
    blog = forms.Field(widget=forms.FileInput, required=True)

    class Meta():
        model = BlogImport
        fields = ('blog',)


from django.conf import settings as d_settings
from django.core.management.base import BaseCommand
from django.core.cache import cache

class Command(BaseCommand):
    """
    Example: python manage.py set_setting site global siteurl http://example.com
    """

    def handle(self, scope=None, scope_category=None, name=None, value=None, **options):
        """
        Set the website theme via theme name
        """
        from tendenci.core.site_settings.models import Setting
        from tendenci.core.site_settings.utils import delete_all_settings_cache

        if scope and scope_category and name and value:
            try:
                setting = Setting.objects.get(
                    name=name,
                    scope=scope,
                    scope_category=scope_category,
                )
                setting.set_value(value)
                setting.save()

            except Setting.DoesNotExist:
                if int(options['verbosity']) > 0:
                    print "We could not update that setting."
            delete_all_settings_cache()

            if name == "sitedisplayname":
                from tendenci.apps.user_groups.models import Group
                from tendenci.apps.entities.models import Entity
                try:
                    entity = Entity.objects.get(pk=1)
                    entity.entity_name = value
                    entity.save()
                except:
                    pass

                try:
                    group = Group.objects.get(pk=1)
                    group.name = value
                    group.label = value
                    group.slug = ''
                    group.save()
                except:
                    pass



from django.core.management.base import BaseCommand


class Command(BaseCommand):
    """
    This command checks membership entries and confirms
    whether they are blank by setting status_detail to 'empty.'
    """
    def handle(self, *args, **options):
        from memberships.models import AppEntry

        # for every membership entry that is associated with a user
        # check for first name last name and email
        for entry in AppEntry.objects.all(user_set__isnull=False):

            fn = entry.first_name.split()
            ln = entry.last_name.split()
            em = entry.email.split()

            if not all(fn, ln, em):
                print entry.pk, entry.status_detail, 'empty'



from django.template import RequestContext
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response, get_object_or_404
from django.contrib.auth.models import User
from django.utils.html import strip_entities, strip_tags

from tendenci.addons.make_payments.forms import MakePaymentForm
from tendenci.addons.make_payments.utils import make_payment_inv_add, make_payment_email_user
from tendenci.addons.make_payments.models import MakePayment
from tendenci.core.site_settings.utils import get_setting
from tendenci.core.base.http import Http403
from tendenci.core.base.utils import tcurrency
from tendenci.core.event_logs.models import EventLog
from tendenci.core.perms.utils import get_notice_recipients

try:
    from tendenci.apps.notifications import models as notification
except:
    notification = None


def add(request, form_class=MakePaymentForm, template_name="make_payments/add.html"):
    if request.method == "POST":
        form = form_class(request.user, request.POST)
        
        if form.is_valid():
            mp = form.save(commit=False)
            # we might need to create a user record if not exist
            if request.user.is_authenticated():
                user = request.user
            else:
                try:
                    user = User.objects.get(email=mp.email)
                except:
                    user = request.user

            if not user.is_anonymous():
                mp.user = user
                mp.creator = user
                mp.creator_username = user.username
            mp.save(user)
            
            # create invoice
            invoice = make_payment_inv_add(user, mp)

            EventLog.objects.log(instance=invoice)

            # updated the invoice_id for mp, so save again
            mp.save(user)

            EventLog.objects.log(instance=mp)
            
            # send notification to administrators
            # get admin notice recipients
            recipients = get_notice_recipients('module', 'payments', 'paymentrecipients')
            if recipients:
                if notification:
                    extra_context = {
                        'mp': mp,
                        'invoice': invoice,
                        'request': request,
                    }
                    notification.send_emails(recipients,'make_payment_added', extra_context)
            
            # email to user 
            email_receipt = form.cleaned_data['email_receipt']
            if email_receipt:
                make_payment_email_user(request, mp, invoice)
            
            # redirect to online payment or confirmation page
            if mp.payment_method == 'cc' or mp.payment_method == 'credit card':
                return HttpResponseRedirect(reverse('payment.pay_online', args=[invoice.id, invoice.guid]))
            else:
                return HttpResponseRedirect(reverse('make_payment.add_confirm', args=[mp.id]))
    else:
        form = form_class(request.user)

        # check for initial payment_amount and clean up
        payment_amount = request.GET.get('payment_amount', 0)
        try:
            payment_amount = float(payment_amount)
        except:
            payment_amount = 0
        if payment_amount > 0:
            form.fields['payment_amount'].initial = payment_amount

        # check for initial comment and clean up
        comments = request.GET.get('comments','')
        if comments:
            comments = strip_tags(comments)
            comments = strip_entities(comments)
            form.fields['comments'].initial = comments

    currency_symbol = get_setting("site", "global", "currencysymbol")
    if not currency_symbol: currency_symbol = "$"
       
    return render_to_response(template_name, {'form':form, 'currency_symbol': currency_symbol}, 
        context_instance=RequestContext(request))
    
def add_confirm(request, id, template_name="make_payments/add_confirm.html"):
    return render_to_response(template_name, context_instance=RequestContext(request))

def view(request, id=None, template_name="make_payments/view.html"):
    mp = get_object_or_404(MakePayment, pk=id)
    if not mp.allow_view_by(request.user): raise Http403

    EventLog.objects.log(instance=mp)

    mp.payment_amount = tcurrency(mp.payment_amount)
    return render_to_response(template_name, {'mp':mp}, 
        context_instance=RequestContext(request))



from django.template import Library
from tendenci.apps.accounts.forms import LoginForm

register = Library()

@register.inclusion_tag("accounts/login_form.html", takes_context=True)
def login_form(context, login_button_value=None):
    form = LoginForm()
    context.update({
        'form': form,
        'login_button_value': login_button_value
    })
    return context




def send_registration_activation_email(user, registration_profile, **kwargs):
    """
        this function sends the activation email to the self registrant.
        modified based on the block in create_inactive_user in registration/models.py
    """
    from django.core.mail import send_mail
    from django.template.loader import render_to_string
    from django.conf import settings
    from tendenci.core.site_settings.utils import get_setting

    event = kwargs.pop('event', None)
    event_url = None
    if event: event_url = event.get_absolute_url()

    site_url = get_setting('site', 'global', 'siteurl')
    subject = render_to_string('registration/activation_email_subject.txt',
                                       { 'site_url': site_url })
    
    # Email subject *must not* contain newlines
    subject = ''.join(subject.splitlines())
    message = render_to_string('registration/activation_email.txt',
                               { 'activation_key': registration_profile.activation_key,
                                 'expiration_days': settings.ACCOUNT_ACTIVATION_DAYS,
                                 'event_url': event_url,
                                 'site_url': site_url })
    
    from_email = get_setting('site', 'global', 'siteemailnoreplyaddress')
    
    send_mail(subject, message, from_email, [user.email])



import os
import urllib2
import time
import uuid
import sys
import re

from dateutil import parser
from datetime import datetime
from BeautifulSoup import BeautifulStoneSoup
from parse_uri import ParseUri

from tendenci.apps.pages.models import Page
from tendenci.addons.articles.models import Article
from tendenci.apps.redirects.models import Redirect
from tendenci.core.files.models import File
from tendenci.apps.wp_importer.models import AssociatedFile

from django.contrib.auth.models import User
from django.conf import settings
from django.template import RequestContext

def replace_short_code(body):
    """
    Replaces shortcodes in the body of an article with appropriate HTML structures.
    """
    # remove CDATA elements
    body = re.sub("^<!\\[CDATA\\[", "", body)
    body = re.sub("\\]\\]>$", "", body)

    body = re.sub("(.*)(\\[caption.*caption=\")(.*)(\"\\])(.*)(<img.*(\"|/| )>)(.*)(\\[/caption\\])(.*)", "\\1\\6<div class=\"caption\">\\3</div>\\10", body)      
    body = re.sub("(.*)(\\[gallery?.*?\\])(.*)", '', body)
    return body

def get_posts(item, uri_parser, user):
    """
    If the given Article has already been created, skip it.
    If not, create Article object and Redirect object.
    """
    alreadyThere = False

    if item.find('link'):
        link = unicode(item.find('link').contents[0])
        slug = uri_parser.parse(link).path.strip('/')
    else:
        # if no slug, grab the post id
        slug = unicode(item.find('wp:post_id').contents[0])

    for article in Article.objects.all():
        if article.slug == slug[:100]:
            alreadyThere = True
            break

    if not alreadyThere:
        title = unicode(item.find('title').contents[0])
        post_id = item.find('wp:post_id').string
        post_id = int(post_id)
        body = unicode(item.find('content:encoded').contents[0])
        body = replace_short_code(body)

        try:
            # There may not be a file associated with a post.
            # If so, catch that error.
            fgroup = AssociatedFile.objects.filter(post_id=post_id)
            for f in fgroup:
                body = correct_media_file_path(body, f.file)

        except:
            pass

        post_date = unicode(item.find('wp:post_date').contents[0])
        #post_dt = datetime.strptime(post_date, '%Y-%m-%d %H:%M:%S')
        post_dt = post_date

        tags_raw = item.findAll('category', domain="post_tag")
        tags_list = []
    
        if tags_raw:
            for tag in tags_raw:
                if len(','.join(tags_list)) + len(tag.string) <= 255:
                    tags_list.append(tag.string[:50])   

        article = {
            'headline': title,
            'guid': str(uuid.uuid1()),
            'slug': slug[:100],
            'body': body,
            'tags': ','.join(tags_list),
            'timezone': 'US/Central',
            'syndicate': True,
            'featured': False,
            'release_dt': post_dt,
            'creator': user,
            'creator_username': user.username,
            'allow_anonymous_view': True,
            'allow_user_view': False,
            'allow_member_view': False,
            'allow_user_edit': False,
            'allow_member_edit': False,
            'owner': user,
            'owner_username': user.username,
            'status': True,
            'status_detail': 'active'
        }

        redirect = {
            'from_url': slug[:100],
            'to_url': os.path.join('articles', slug[:100])
        }

        a = Article(**article)
        a.save()
        r = Redirect(**redirect)
        r.save()
    
def get_pages(item, uri_parser, user):
    """
    Find each item marked "page" in items.
    If that Page has already been created, do nothing.
    If not, create Page object.
    """
    alreadyThere = False
    link = unicode(item.find('link').contents[0])
    slug = uri_parser.parse(link).path.strip('/')

    for page in Page.objects.all():
        if page.slug == slug[:100]:
            alreadyThere = True
            break
   
    if not alreadyThere:
        title = unicode(item.find('title').contents[0])
        post_id = item.find('wp:post_id').string
        post_id = int(post_id)
        body = unicode(item.find('content:encoded').contents[0])
        body = replace_short_code(body)
        try:
            fgroup = AssociatedFile.objects.filter(post_id=post_id)
            for f in fgroup:
                body = correct_media_file_path(body, f.file)
                
        except:
            pass
        
        page = {
            'title': title,
            'guid': str(uuid.uuid1()),
            'slug': slug[:100],
            'content': body,
            #'timezone': 'US/Central',
            'syndicate': True,
            #'featured': False,
            #'create_dt': datetime.now(), 
            'creator': user,
            'creator_username': user.username,
            'allow_anonymous_view': True,
            'allow_user_view': False,
            'allow_member_view': False,
            'allow_user_edit': False,
            'allow_member_edit': False,
            'owner': user,
            'owner_username': user.username,
            'status': True,
            'status_detail': 'active'
        }

        p = Page(**page)
        p.save()

def get_media(item, uri_parser, user):
    """
    Find any URL contained in an "attachment."
    If that File has already been created, skip it.
    If not, go to the URL, and save the media there as a File.
    Loop through Articles and Pages and replace links.
    """
    media_url_in_attachment = item.find('wp:attachment_url').string
    media_url = uri_parser.parse(media_url_in_attachment).file
    media_url = os.path.join(settings.MEDIA_ROOT, media_url)

    post_id = item.find('wp:post_parent').string
    post_id = int(post_id)

    alreadyThere = False

    for url in File.objects.all():
        if media_url == url.file:
            alreadyThere = True
            # This assignment will make sure the file gets replaced in the HTML even if
            # it's an old file that already exists in the database.
            new_media = url
            break

    if not alreadyThere:
        source = urllib2.urlopen(media_url_in_attachment).read()

        with open(media_url, 'wb') as f:
            f.write(source)
            file_path = f.name

        new_media = File(guid=unicode(uuid.uuid1()), file=file_path, creator=user, owner=user)
        new_media.save()

    temporary = AssociatedFile(post_id=post_id, file=new_media)
    temporary.save()
    

def correct_media_file_path(body, file):
    """
    Replace an instance of the given file's URL in the given HTML with a file path.
    """
    match = re.search("(.*)(http://.*\\/?\\/\\b\\w+\\/)((\\S+)(\\-\\d+x.*?\\S*.*)|(\\S+.*?\\S+))(\\.\\S+)(\\\".*)", body)
    if match:
        match.group()

        if match.group(4) == None and file.basename() == match.group(6) + match.group(7):
            # if the file is unsized
            body = re.sub("(.*)(http://.*\\/?\\/\\b\\w+\\/)(" + re.escape(file.basename()) + ".*?)(\\\".*)", "\\1/files/" + str(file.pk) + "/\\4", body)
        elif match.group(6) == None and file.basename() == match.group(4) + match.group(7):
            # if the file is sized
            body = re.sub("(.*)(http://.*\\/?\\/\\b\\S+\\/)(" + re.escape(match.group(4) + match.group(5) + match.group(7)) + ".*?)(\\\".*)", "\\1/files/" + str(file.pk) + "/\\4", body)

    return body
    



from django.utils.translation import ugettext_lazy as _
from django.utils import simplejson as json
from django.views.decorators.csrf import csrf_exempt
from django.contrib.auth.decorators import login_required
from django.shortcuts import get_object_or_404, redirect
from django.template import RequestContext
from django.contrib import messages
from django.http import HttpResponse
from django.conf import settings

from tendenci.core.base.http import Http403
from tendenci.core.perms.utils import has_perm, update_perms_and_save, get_query_filters
from tendenci.core.event_logs.models import EventLog
from tendenci.core.theme.shortcuts import themed_response as render_to_response
from tendenci.core.exports.utils import run_export_task

from tendenci.apps.discounts.models import Discount, DiscountUse
from tendenci.apps.discounts.forms import DiscountForm, DiscountCodeForm, DiscountHandlingForm

@login_required
def search(request, template_name="discounts/search.html"):
    if not has_perm(request.user, 'discounts.view_discount'):
        raise Http403

    filters = get_query_filters(request.user, 'discounts.view_discount')
    discounts = Discount.objects.filter(filters).distinct()
    query = request.GET.get('q', None)
    if query:
        discounts = discounts.filter(discount_code__icontains=query)

    EventLog.objects.log()

    return render_to_response(
        template_name,
        {'discounts':discounts},
        context_instance=RequestContext(request)
    )

@login_required    
def detail(request, id, template_name="discounts/detail.html"):
    discount = get_object_or_404(Discount, id=id)
    
    if not has_perm(request.user, 'discounts.view_discount', discount):
        raise Http403

    EventLog.objects.log(instance=discount)

    return render_to_response(
        template_name, 
        {'discount':discount},
        context_instance=RequestContext(request)
    )

@login_required
def add(request, form_class=DiscountForm, template_name="discounts/add.html"):
    if not has_perm(request.user, 'discounts.add_discount'):
        raise Http403
    
    if request.method == "POST":
        form = form_class(request.POST, user=request.user)
        if form.is_valid():
            discount = form.save(commit=False)
            discount = update_perms_and_save(request, form, discount)

            messages.add_message(request, messages.SUCCESS, 'Successfully added %s' % discount)
            return redirect('discount.detail', id=discount.id)
    else:
        form = form_class(user=request.user)
        
    return render_to_response(
        template_name,
        {'form':form},
        context_instance=RequestContext(request),
    )

@login_required
def edit(request, id, form_class=DiscountForm, template_name="discounts/edit.html"):
    discount = get_object_or_404(Discount, id=id)
    if not has_perm(request.user, 'discounts.change_discount', discount):
        raise Http403
    
    if request.method == "POST":
        form = form_class(request.POST, instance=discount, user=request.user)
        if form.is_valid():
            discount = form.save(commit=False)
            discount = update_perms_and_save(request, form, discount)

            messages.add_message(request, messages.SUCCESS, 'Successfully updated %s' % discount)
            return redirect('discount.detail', id=discount.id)
    else:
        form = form_class(instance=discount, user=request.user)

    return render_to_response(
        template_name,
        {
            'form': form,
            'discount': discount,
        },
        context_instance=RequestContext(request),
    )


@login_required
def delete(request, id, template_name="discounts/delete.html"):
    discount = get_object_or_404(Discount, pk=id)

    if not has_perm(request.user, 'discounts.delete_discount', discount):
        raise Http403

    if request.method == "POST":
        messages.add_message(request, messages.SUCCESS, 'Successfully deleted %s' % discount)
        discount.delete()

        return redirect('discounts')

    return render_to_response(template_name, {'discount': discount},
        context_instance=RequestContext(request))


@csrf_exempt
def discounted_price(request, form_class=DiscountCodeForm):
    if request.method == 'POST':
        form = form_class(request.POST)
        if form.is_valid():
            return HttpResponse(json.dumps(
                {
                    "error": False,
                    "price": unicode(form.new_price()[0]),
                    "discount": unicode(form.new_price()[1]),
                    "message": "Your discount of $ %s has been added." % unicode(form.new_price()[1]),
                }), mimetype="text/plain")
        return HttpResponse(json.dumps(
            {
                "error": True,
                "message": "This is not a valid discount code.",
            }), mimetype="text/plain")
    else:
        form = form_class()
    return HttpResponse(
        "<form action='' method='post'>" + form.as_p() + "<input type='submit' value='check'> </form>",
        mimetype="text/html")


@csrf_exempt
def discounted_prices(request, form_class=DiscountHandlingForm):
    if request.method == 'POST':
        form = form_class(request.POST)
        if form.is_valid():
            price_list, discount_total, discount_list, msg = form.get_discounted_prices()
            total = sum(price_list)
            new_prices = ';'.join([str(price) for price in price_list])
            if sum(discount_list) > 0:
                discount_detail = '(%s)' % (', '.join(([str(price) for price in discount_list if price >0])))
            else:
                discount_detail = ''
            return HttpResponse(json.dumps(
                {
                    "error": False,
                    "prices": unicode(new_prices),
                    "discount_total": unicode(discount_total),
                    "total": unicode(total),
                    "message": "%sYour discount of $%s %s has been added." % (unicode(msg), unicode(discount_total), 
                                                                               discount_detail),
                }), mimetype="text/plain")
        return HttpResponse(json.dumps(
            {
                "error": True,
                "message": "This is not a valid discount code.",
            }), mimetype="text/plain")
    else:
        form = form_class()
    return HttpResponse(
        "<form action='' method='post'>" + form.as_p() + "<input type='submit' value='check'> </form>",
        mimetype="text/html")


@login_required
def export(request, template_name="discounts/export.html"):
    """Export Discounts"""

    if not request.user.is_superuser:
        raise Http403

    if request.method == 'POST':
        # initilize initial values
        fields = [
            'discount_code',
            'start_dt',
            'end_dt',
            'never_expires',
            'value',
            'cap',
        ]
        export_id = run_export_task('discounts', 'discount', fields)
        EventLog.objects.log()
        return redirect('export.status', export_id)

    return render_to_response(template_name, {
    }, context_instance=RequestContext(request))



from optparse import make_option
from pysolr import Solr
import sys

from django.core.management.base import BaseCommand
from django.conf import settings


class Command(BaseCommand):
    help = "Clears out the search index completely."
    base_options = (
        make_option('--noinput', action='store_false', dest='interactive', default=True,
            help='If provided, no prompts will be issued to the user and the data will be wiped out.'
        ),
    )
    option_list = BaseCommand.option_list + base_options
    
    def handle(self, **options):
        """Clears out the search index completely."""
        # Cause the default site to load.
        from haystack import site
        self.verbosity = int(options.get('verbosity', 1))
        self.scratch = options.get('scratch', None)
        
        if options.get('interactive', True):
            print
            print "WARNING: This will irreparably remove EVERYTHING from your search index."
            print "Your choices after this are to restore from backups or rebuild via the `rebuild_index` command."
            
            yes_or_no = raw_input("Are you sure you wish to continue? [y/N] ")
            print
            
            if not yes_or_no.lower().startswith('y'):
                print "No action taken."
                sys.exit()
        
        if self.verbosity >= 1:
            print "Removing all documents from your index because you said so."
        
        from haystack import backend
        
        sb = backend.SearchBackend()

        if self.scratch:
            timeout = getattr(settings, 'HAYSTACK_SOLR_TIMEOUT', 10)
            sb.conn = Solr(self.scratch, timeout=timeout)

        sb.clear()
        
        if self.verbosity >= 1:
            print "All documents removed."



import os
import simplejson as json

from django.conf import settings as django_settings
from django.core.management.base import BaseCommand, CommandError

from tendenci.core.site_settings.models import Setting


class Command(BaseCommand):
    """
    Update site settings in the database. It reads the settings.json 
    under each installed app or apps specified in the arguments,
    and add or update the settings accordingly.
    
    Usage:
        manage.py update_settings <appname appname ...> 
        
    Example:   
        manage.py update_settings articles plugins.donations
        
    If no appname specified, it updates for ALL installed apps. 

    Json required fields (for lookups):
        `scope`
        `scope_category`
        `name`

    Json columns that cannot be updated:
        `value`: It would just be mean to override a
                 clients setting value

    Json format:
        [
          {
            "name": "",
            "label": "",
            "description": "",
            "data_type": "",
            "default_value": "",
            "input_type": "",
            "input_value": "",
            "client_editable": "",
            "store": "",
            "scope": "",
            "scope_category": "",
          }
        ]

    Json example:
        [
          {
            "name": "enabled",
            "label": "enabled",
            "description": "Module is enabled or not.",
            "data_type": "boolean",
            "default_value": "true",
            "input_type": "select",
            "input_value": "true, false",
            "client_editable": "1",
            "store": "1",
            "scope": "module",
            "scope_category": "memberships",
          }
        ]

    Json field definitions:
        `name`: The machine name. No spaces or special characters.
              Remember that this is what the python code uses to
              find the setting

        `label`: The human readable version of 'name'

        `description`: A non-html or html description of the setting.
                       Refer to the 'site_settings_setting' table for examples

        `data_type`: boolean or string

        `default_value`: the original value

        `input_type`: select or text (used by the autogenerated interface)

        `input_value`: comma delimited list or just a string

        `store`: boolean value. Tell the system whether or not to
               cache the setting

        `scope`: site or module

        `scope_category`: this is the module name the settings belongs too.
                        refer to django contenttypes app_label. If this
                        is a settings that doesn't apply to a django
                        application use 'global'
    """
    help = 'Update settings in the site_settings_setting table'

    def update_settings(self, settings, verbosity=1):
        """
        Loop through the settings and add or update them
        """
        required_keys = [
            'scope',
            'scope_category',
            'name'
        ]
        for setting in settings:
            # check the required fields
            req_list = [k for k in setting.keys() if k in required_keys]
            if len(req_list) != len(required_keys):
                print 'Setting does not have the required fields ... skipping.'
                continue

            try:
                current_setting = Setting.objects.filter(**{
                    'name': setting['name'],
                    'scope': setting['scope'],
                    'scope_category': setting['scope_category']
                })[0]
            except:
                current_setting = None

            # update the setting
            if (current_setting):
                # skip the value for the existing setting
                if setting.has_key('value'):
                    del setting['value']
                current_setting.__dict__.update(setting)
                current_setting.save()
                print '%s (%s)  - updated.' % (
                    setting['name'],
                    setting['scope_category']
                )
            else:
                # insert
                new_setting = Setting(**setting)
                new_setting.save()
                #if verbosity >= 2:
                print '%s (%s)  - added.' % (
                    setting['name'],
                    setting['scope_category']
                )
                

    def handle(self, *args, **options):
        try:
            verbosity = int(options['verbosity'])
        except:
            verbosity = 1

        if args:
            appnames = args
        else:
            appnames = django_settings.INSTALLED_APPS
            # exclude django native apps
            appnames = [app for app in appnames if not app.startswith('django.')]

        for appname in appnames:
            print
            print 'Processing for %s ...' % appname
            if appname.startswith('addons.') or appname.startswith('themes.'):
                json_file = os.path.abspath(os.path.join(
                                django_settings.PROJECT_ROOT,
                                '/'.join(appname.split('.')),
                                'settings.json'
                            ))
            else:
                if appname.find('.') != -1:
                    appname = appname.replace('.', '/')
                json_file = os.path.abspath(os.path.join(
                                django_settings.TENDENCI_ROOT,'..',
                                '/'.join(appname.split('.')),
                                'settings.json'
                            ))
            if os.path.isfile(json_file):
                with open(json_file, 'r') as f:
                    try:
                        settings = json.loads(f.read())
                    except ValueError as e:
                        print "Error updating setting for %s/settings.json" % appname
                        print e
                        continue
      
                if settings:
                    self.update_settings(settings, verbosity=verbosity)



from django.contrib.auth.models import User
from tastypie.validation import CleanedDataFormValidation

class TendenciValidation(CleanedDataFormValidation):
    """Validation that makes use of a given Form.
    The Form is assumed to be a ModelForm of a TendenciBaseModel subclass
    """
    
    def is_valid(self, bundle, request=None):
        """Validate the bundle with the given form.
        Creator and Owner fields and defaulted to be the user of 
        the apikey if they were not specified.
        """
        
        data = bundle.data
        # Ensure we get a bound Form, regardless of the state of the bundle.
        if data is None:
            data = {}
        
        if request:
            user = User.objects.get(username=request.GET.get('username'))
            if not data.get('creator', None):
                data['creator'] = user.pk
                data['creator_username'] = user.username
            if not data.get('owner', None):
                data['owner'] = user.pk
                data['owner_username'] = user.username
        
        form = self.form_class(data, instance=bundle.obj, request=request)
        
        if form.is_valid():
            # We're different here & relying on having a reference to the same
            # bundle the rest of the process is using.
            bundle.data = form.cleaned_data
            return {}
        
        # The data is invalid. Let's collect all the error messages & return
        # them.
        return form.errors



from django import forms

from tendenci.apps.redirects.models import Redirect

class RedirectForm(forms.ModelForm):
    def clean_from_url(self):
        value = self.cleaned_data['from_url']
        value = value.strip('/')
        return value

    def clean_to_url(self):
        value = self.cleaned_data['to_url']
        value = value.strip('/')
        return value
        
    class Meta:
        model = Redirect




from django import forms
from django.forms import widgets

from tendenci.addons.help_files.models import Request, HelpFile, Topic
from tinymce.widgets import TinyMCE
from captcha.fields import CaptchaField
from tendenci.core.perms.forms import TendenciBaseForm
from tendenci.apps.user_groups.models import Group

class RequestForm(forms.ModelForm):
    captcha = CaptchaField()
    class Meta:
        model = Request

class HelpFileAdminForm(TendenciBaseForm):
    answer = forms.CharField(required=False,
        widget=TinyMCE(attrs={'style':'width:100%'}, 
        mce_attrs={'storme_app_label':HelpFile._meta.app_label, 
        'storme_model':HelpFile._meta.module_name.lower()}))

    status_detail = forms.ChoiceField(choices=(('draft','Draft'),('active','Active')))
    
    group = forms.ModelChoiceField(queryset=Group.objects.filter(status=True, status_detail="active"), required=True, empty_label=None)
    
    class Meta:
        model = HelpFile
        fields = (
            'question',
            'slug',
            'answer',
            'group',
            'level',
            'topics',
            'is_faq',
            'is_featured',
            'is_video',
            'syndicate',
            'allow_anonymous_view',
            'user_perms',
            'member_perms',
            'group_perms',
            'status',
            'status_detail',
        )

    def __init__(self, *args, **kwargs): 
        super(HelpFileAdminForm, self).__init__(*args, **kwargs)
        if self.instance.pk:
            self.fields['answer'].widget.mce_attrs['app_instance_id'] = self.instance.pk
        else:
            self.fields['answer'].widget.mce_attrs['app_instance_id'] = 0


class HelpFileForm(TendenciBaseForm):
    answer = forms.CharField(required=False,
        widget=TinyMCE(attrs={'style':'width:100%'}, 
        mce_attrs={'storme_app_label':HelpFile._meta.app_label, 
        'storme_model':HelpFile._meta.module_name.lower()}))

    status_detail = forms.ChoiceField(
        choices=(('draft','Draft'),('active','Active')))

    #topics = forms.MultipleChoiceField(required=True, widget=widgets.CheckboxSelectMultiple())

    class Meta:
        model = HelpFile
        fields = (
            'question',
            'slug',
            'answer',
            'level',
            'topics',
            'is_faq',
            'is_featured',
            'is_video',
            'syndicate',
            'allow_anonymous_view',
            'user_perms',
            'member_perms',
            'group_perms',
            'status',
            'status_detail',
        )

        fieldsets = [('Help File Information', {
                      'fields': ['question',
                                 'slug',
                                 'answer',
                                 'level',
                                 'topics',
                                 ],
                      'legend': ''
                      }),
                      ('Flags', {
                      'fields': ['is_faq',
                                 'is_featured',
                                 'is_video',
                                 ],
                        'classes': ['flags'],
                      }),
                      ('Permissions', {
                      'fields': ['allow_anonymous_view',
                                 'user_perms',
                                 'member_perms',
                                 'group_perms',
                                 ],
                      'classes': ['permissions'],
                      }),
                     ('Administrator Only', {
                      'fields': ['syndicate',
                                 'status',
                                 'status_detail'], 
                      'classes': ['admin-only'],
                    })]

    def __init__(self, *args, **kwargs):
        super(HelpFileForm, self).__init__(*args, **kwargs)
        if self.instance.pk:
            self.fields['answer'].widget.mce_attrs['app_instance_id'] = self.instance.pk
        else:
            self.fields['answer'].widget.mce_attrs['app_instance_id'] = 0

        if not self.user.profile.is_superuser:
            if 'status' in self.fields: self.fields.pop('status')
            if 'status_detail' in self.fields: self.fields.pop('status_detail')
            if 'allow_anonymous_view' in self.fields: self.fields.pop('allow_anonymous_view')
            if 'user_perms' in self.fields: self.fields.pop('user_perms')
            if 'member_perms' in self.fields: self.fields.pop('member_perms')
            if 'group_perms' in self.fields: self.fields.pop('group_perms')
            if 'syndicate' in self.fields: self.fields.pop('syndicate')



from optparse import make_option
from django.core.management.base import BaseCommand
from django.core.exceptions import ObjectDoesNotExist 

class Command(BaseCommand):
    """
    Get/Delete list of contributors, non-contributors and total users
    """

    option_list = BaseCommand.option_list + (
        make_option('-d', '--delete',
        action='store_true',
        default=False,
        help='Delete non contributing users'),
    )

    def handle(self, *args, **options):
        from django.contrib.auth.models import User

        contribs = []
        
        try:
            from tendenci.addons.directories.models import Directory
            for directory in Directory.objects.all():
                contribs.append(directory.creator)
                contribs.append(directory.owner)
        except ImportError:
            pass
        
        try:
            from tendenci.addons.articles.models import Article
            for article in Article.objects.all():
                contribs.append(article.creator)
                contribs.append(article.owner)
        except ImportError:
            pass
        
        try:
            from tendenci.addons.events.models import Event
            for event in Event.objects.all():
                contribs.append(event.creator)
                contribs.append(event.owner)
        except ImportError:
            pass

        try:
            from tendenci.addons.photos.models import Photo
            for photo in Photo.objects.all():
                contribs.append(photo.creator)
                contribs.append(photo.owner)
        except ImportError:
            pass

        try:
            from tendenci.apps.pages.models import Page
            for page in Page.objects.all():
                contribs.append(page.creator)
                contribs.append(page.owner)
        except ImportError:
            pass

        try:
            from tendenci.addons.news.models import News
            for news in News.objects.all():
                contribs.append(news.creator)
                contribs.append(news.owner)
        except ImportError:
            pass

        contribs = list(set(contribs))  # remove duplicates
        slackers = User.objects.exclude(username__in=[c.username for c in contribs])

        print 'contribs', len(contribs)
        print 'slackers', slackers.count()
        print 'everyone', User.objects.count()
        print 'Pass the -d or --delete fn to delete no contributors'

        delete = options['delete']

        if delete:
            from django.db import connections, DEFAULT_DB_ALIAS, IntegrityError
            using = options.get('database', DEFAULT_DB_ALIAS)
            connection = connections[using]
            cursor = connection.cursor()

            cursor.execute('SET FOREIGN_KEY_CHECKS=0;')
            for slacker in slackers:
                try:
                    print slacker
                    slacker.delete()
                except IntegrityError as e:
                    print 'Integrity Error deleting', slacker

            cursor.execute('SET FOREIGN_KEY_CHECKS=1;')


from django.contrib import admin
from tendenci.addons.campaign_monitor.models import Campaign, Template
from tendenci.addons.campaign_monitor.forms import CampaignForm, TemplateForm


class TemplateAdmin(admin.ModelAdmin):
    form = TemplateForm


class CampaignAdmin(admin.ModelAdmin):
    form = CampaignForm

# admin.site.register(Template, TemplateAdmin)
# admin.site.register(Campaign, CampaignAdmin)



from tastypie import fields
from tendenci.apps.entities.models import Entity
from tendenci.core.api_tasty.resources import TendenciResource

class EntityResource(TendenciResource):
    class Meta(TendenciResource.Meta):
        queryset = Entity.objects.all()
        resource_name = 'entity'
        list_allowed_methods = ['get',]
        detail_allowed_methods = ['get',]



from django.conf.urls.defaults import patterns, include
from tendenci.apps.pluginmanager.utils import get_addons


def get_url_patterns():
    from django.conf import settings
    items = []
    addons = get_addons(settings.INSTALLED_APPS)
    for addon in addons:
        items.append((r'', include('%s.urls' % addon,)))
    return patterns('', *items)
    pass



from django.conf import settings
from django.utils.translation import ugettext_noop as _
from django.db.models.signals import post_syncdb

from south.signals import post_migrate

from tendenci.apps.notifications import models as notification

def create_notice_types(app, **kwargs):
    notification.create_notice_type(
        'event_added',
        _('Event Added'),
        _('An event has been added'))

    notification.create_notice_type(
        'event_edited',
        _('Event Edited'),
        _('An event has been edited'))

    notification.create_notice_type(
        'event_deleted',
        _('Event Deleted'),
        _('An event has been deleted'))

    notification.create_notice_type(
        'event_registration_confirmation',
        _('Event Registration Confirmation'),
        _('The email you receive confirming your registration'))

    notification.create_notice_type(
        'event_registration_cancelled',
        _('Event Registration Cancelled'),
        _('Notify administrators that someone has cancelled their event registration'))
        
    notification.create_notice_type(
        'event_registration_end_recap',
        _('Recap of end of event registration'),
        _('Notify administrators that registration for the event has ended.'))

post_syncdb.connect(create_notice_types, sender=notification)
post_migrate.connect(create_notice_types, sender=notification)



import uuid
from datetime import datetime
from django.db import models
from tendenci.apps.user_groups.models import Group
from tendenci.apps.user_groups.utils import get_default_group
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes import generic

from tagging.fields import TagField
from tendenci.core.base.fields import SlugField
from timezones.fields import TimeZoneField
from tendenci.core.perms.models import TendenciBaseModel
from tendenci.core.perms.object_perms import ObjectPermission
from tendenci.core.categories.models import CategoryItem
from tendenci.addons.articles.managers import ArticleManager
from tinymce import models as tinymce_models
from tendenci.core.meta.models import Meta as MetaTags
from tendenci.addons.articles.module_meta import ArticleMeta


class Article(TendenciBaseModel):
    guid = models.CharField(max_length=40)
    slug = SlugField(_('URL Path'), unique=True)
    timezone = TimeZoneField(_('Time Zone'))
    headline = models.CharField(max_length=200, blank=True)
    summary = models.TextField(blank=True)
    body = tinymce_models.HTMLField()
    source = models.CharField(max_length=300, blank=True)
    first_name = models.CharField(_('First Name'), max_length=100, blank=True)
    last_name = models.CharField(_('Last Name'), max_length=100, blank=True)
    phone = models.CharField(max_length=50, blank=True)
    fax = models.CharField(max_length=50, blank=True)
    email = models.CharField(max_length=120, blank=True)
    website = models.CharField(max_length=300, blank=True)
    release_dt = models.DateTimeField(_('Release Date/Time'), null=True, blank=True)
    syndicate = models.BooleanField(_('Include in RSS feed'), default=True)
    featured = models.BooleanField()
    design_notes = models.TextField(_('Design Notes'), blank=True)
    group = models.ForeignKey(Group, null=True, default=get_default_group, on_delete=models.SET_NULL)
    tags = TagField(blank=True)

    # for podcast feeds
    enclosure_url = models.CharField(_('Enclosure URL'), max_length=500, blank=True)
    enclosure_type = models.CharField(_('Enclosure Type'), max_length=120, blank=True)
    enclosure_length = models.IntegerField(_('Enclosure Length'), default=0)

    not_official_content = models.BooleanField(_('Official Content'), blank=True)

    # html-meta tags
    meta = models.OneToOneField(MetaTags, null=True)

    categories = generic.GenericRelation(CategoryItem,
                                          object_id_field="object_id",
                                          content_type_field="content_type")
    perms = generic.GenericRelation(ObjectPermission,
                                          object_id_field="object_id",
                                          content_type_field="content_type")

    objects = ArticleManager()

    class Meta:
        permissions = (("view_article", "Can view article"),)
        verbose_name = "Article"
        verbose_name_plural = "Articles"

    def get_meta(self, name):
        """
        This method is standard across all models that are
        related to the Meta model.  Used to generate dynamic
        methods coupled to this instance.
        """
        return ArticleMeta().get_meta(self, name)

    @models.permalink
    def get_absolute_url(self):
        return ("article", [self.slug])

    @models.permalink
    def get_version_url(self, hash):
        return ("article.version", [hash])

    def __unicode__(self):
        return self.headline

    def save(self, *args, **kwargs):
        if not self.id:
            self.guid = str(uuid.uuid1())
        super(Article, self).save(*args, **kwargs)

    def age(self):
        return datetime.now() - self.create_dt

    @property
    def category_set(self):
        items = {}
        for cat in self.categories.select_related('category__name', 'parent__name'):
            if cat.category:
                items["category"] = cat.category
            elif cat.parent:
                items["sub_category"] = cat.parent
        return items



from django import forms
from django.contrib.auth.models import User
from tastypie.models import ApiKey
from tendenci.core.perms.models import TendenciBaseModel

class ApiKeyForm(forms.ModelForm):
    """
    From for setting up ApiKeys for superusers.
    """
    
    class Meta:
        model = ApiKey
        exclude = ('created', 'key')
        
    def clean_user(self):
        user = self.cleaned_data['user']
        if not user.profile.is_superuser:
            raise forms.ValidationError('This user is not a superuser.')
        return user

class TendenciForm(forms.ModelForm):
    """Form that allows owner and creator fields to be editable
    """
    creator = forms.ModelChoiceField(queryset=User.objects.all())
    owner = forms.ModelChoiceField(queryset=User.objects.all())

    class Meta:
        model = TendenciBaseModel



from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.decorators import login_required
from django.shortcuts import get_object_or_404, redirect
from django.template import RequestContext
from django.contrib import messages
from django.http import HttpResponse, HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.forms.models import modelformset_factory
from django.utils import simplejson as json
from django.conf import settings
from django.db.models import Q

from tendenci.core.theme.shortcuts import themed_response as render_to_response
from tendenci.core.base.http import Http403
from tendenci.core.event_logs.models import EventLog
from tendenci.core.site_settings.utils import get_setting
from tendenci.core.perms.utils import has_perm, update_perms_and_save, get_query_filters, has_view_perm
from tendenci.apps.pages.models import Page
from tendenci.core.exports.utils import run_export_task

from tendenci.apps.navs.models import Nav, NavItem
from tendenci.apps.navs.forms import NavForm, PageSelectForm, ItemForm
from tendenci.apps.navs.utils import cache_nav

@login_required
def search(request, template_name="navs/search.html"):
    query = request.GET.get('q', None)

    filters = get_query_filters(request.user, 'navs.view_nav')
    navs = Nav.objects.filter(filters).distinct()
    if query:
        navs = navs.filter(Q(title__icontains=query)|Q(description__icontains=query))

    EventLog.objects.log()

    return render_to_response(
        template_name,
        {'navs':navs},
        context_instance=RequestContext(request)
    )

@login_required
def detail(request, id, template_name="navs/detail.html"):
    nav = get_object_or_404(Nav, id=id)
    
    if not has_view_perm(request.user, 'navs.view_nav', nav):
        raise Http403

    EventLog.objects.log(instance=nav)

    return render_to_response(
        template_name,
        {'current_nav':nav},
        context_instance=RequestContext(request),
    )

@login_required
def add(request, form_class=NavForm, template_name="navs/add.html"):
    if not has_perm(request.user, 'navs.add_nav'):
        raise Http403

    if request.method == "POST":
        form = form_class(request.POST, user=request.user)
        if form.is_valid():
            nav = form.save(commit=False)
            nav = update_perms_and_save(request, form, nav)

            messages.add_message(request, messages.SUCCESS, 'Successfully added %s' % nav)
            return redirect('navs.edit_items', id=nav.id)
    else:
        form = form_class(user=request.user)

    return render_to_response(
        template_name,
        {'form':form},
        context_instance=RequestContext(request),
    )

@login_required
def edit(request, id, form_class=NavForm, template_name="navs/edit.html"):
    nav = get_object_or_404(Nav, id=id)
    if not has_perm(request.user, 'navs.change_nav', nav):
        raise Http403
    
    if request.method == "POST":
        form = form_class(request.POST, instance=nav, user=request.user)
        if form.is_valid():
            nav = form.save(commit=False)
            nav = update_perms_and_save(request, form, nav)
            cache_nav(nav)

            messages.add_message(request, messages.SUCCESS, 'Successfully updated %s' % nav)
            return redirect('navs.edit_items', id=nav.id)
    else:
        form = form_class(user=request.user, instance=nav)

    return render_to_response(
        template_name,
        {'form':form, 'current_nav':nav},
        context_instance=RequestContext(request),
    )

@login_required
def edit_items(request, id, template_name="navs/nav_items.html"):
    nav = get_object_or_404(Nav, id=id)
    if not has_perm(request.user, 'navs.change_nav', nav):
        raise Http403

    ItemFormSet = modelformset_factory(NavItem,
                        form=ItemForm,
                        extra=0,
                        can_delete=True)
    page_select = PageSelectForm()

    if request.method == "POST":
        formset = ItemFormSet(request.POST)
        if formset.is_valid():
            #delete old nav items
            nav.navitem_set.all().delete()
            items = formset.save(commit=False)
            # update or create nav items
            for item in items:
                item.nav = nav
                item.save()
            cache_nav(nav)
            messages.add_message(request, messages.SUCCESS, 'Successfully updated %s' % nav)

            EventLog.objects.log(instance=nav)

            redirect_to = request.REQUEST.get('next', '')
            if redirect_to:
                return HttpResponseRedirect(redirect_to)
            else:
                return redirect('navs.detail', id=nav.id)
    else:
        formset = ItemFormSet(queryset=nav.navitem_set.all().order_by('ordering'))

    return render_to_response(
        template_name,
        {'page_select':page_select, 'formset':formset, 'current_nav':nav},
        context_instance=RequestContext(request),
    )

@login_required
def delete(request, id, template_name="navs/delete.html"):
    nav = get_object_or_404(Nav, pk=id)

    if has_perm(request.user,'navs.delete_nav'):
        if request.method == "POST":
            messages.add_message(request, messages.SUCCESS, 'Successfully deleted %s' % nav)

            nav.delete()
            return HttpResponseRedirect(reverse('navs.search'))
    
        return render_to_response(template_name, {'current_nav': nav},
            context_instance=RequestContext(request))
    else:
        raise Http403

@login_required
def page_select(request, form_class=PageSelectForm):
    if not request.user.profile.is_superuser:
        raise Http403

    if request.method=="POST":
        form = form_class(request.POST)
        if form.is_valid():
            pages = form.cleaned_data['pages']
            infos = []
            for page in pages:
                infos.append({
                    "url": page.get_absolute_url(),
                    "label": page.title,
                    "id": page.id,
                })
            return HttpResponse(json.dumps({
                "pages": infos,
            }), mimetype="text/plain")
    return HttpResponse(json.dumps({
                "error": True
            }), mimetype="text/plain")

@login_required
def export(request, template_name="navs/export.html"):
    """Export Navs"""
    if not request.user.is_superuser:
        raise Http403

    if request.method == 'POST':
        export_id = run_export_task('navs', 'nav', [])

        EventLog.objects.log()

        return redirect('export.status', export_id)

    return render_to_response(template_name, {
    }, context_instance=RequestContext(request))



from datetime import datetime
from django.shortcuts import render_to_response, redirect, get_object_or_404
from django.contrib.auth.decorators import login_required
from django.template import RequestContext

from tendenci.core.base.http import Http403
from tendenci.core.event_logs.models import EventLog
from tendenci.core.exports.models import Export


@login_required
def status(request, export_id, template_name='exports/export_status.html'):
    """Checks if an export is completed.
    """

    if not request.user.is_superuser:
        raise Http403

    export = get_object_or_404(Export, pk=export_id)

    return render_to_response(template_name, {
        'export': export,
        'datetime': datetime,
    }, context_instance=RequestContext(request))


@login_required
def download(request, export_id):
    """Returns the export file if it exists"""

    if not request.user.is_superuser:
        raise Http403

    export = get_object_or_404(Export, pk=export_id)

    EventLog.objects.log(instance=export)

    if export.status == "completed":
        response = export.result
        return response

    return redirect("export.status", export_id)



from haystack import indexes
from haystack import site

from django.utils.html import strip_tags, strip_entities

from tendenci.core.perms.indexes import TendenciBaseSearchIndex
from tendenci.core.perms.object_perms import ObjectPermission
from tendenci.apps.discounts.models import Discount

class DiscountIndex(TendenciBaseSearchIndex):
    discount_code = indexes.CharField(model_attr='discount_code')
    start_dt = indexes.DateTimeField(model_attr='start_dt')
    end_dt = indexes.DateTimeField(model_attr='end_dt')
    never_expires = indexes.BooleanField(model_attr='never_expires')
    value = indexes.DecimalField(model_attr='value')
    cap = indexes.IntegerField(model_attr='cap')
    
    num_of_uses = indexes.IntegerField()
    
    def prepare_num_of_uses(self, obj):
        return obj.num_of_uses()

site.register(Discount, DiscountIndex)



from django.core.management.base import BaseCommand
from django.conf import settings
from django.db.models import get_models, get_app
from django.db.models import ForeignKey, OneToOneField


class Command(BaseCommand):
    """
    Generate a list of tables and sort them based on the relations
    for db migration from mysql to postgresql.

    Usage: manage.py list_tables
    """

    def handle(self, *args, **options):
        apps = []
        models_d = {}
        tables_list = []
        for app in settings.INSTALLED_APPS:
            try:
                app_label = app.split('.')[-1]
                apps.append(app_label)
            except:
                # No models, no problem.
                pass

        for app_label in apps:
            # skip the legacy
            if app_label in ['legacy']:
                continue
            # skip the social_auth if not set
            if not getattr(settings, 'SOCIAL_AUTH_USER_MODEL', None):
                if app_label in  ['social_auth']:
                    continue
            try:
                app = get_app(app_label)
            except:
                app = None
            if app:
                for model in get_models(app, include_auto_created=True):
                    models_d[model._meta.db_table] = model
                    tables_list.append(model._meta.db_table)

        tables_list.remove('mig_help_files_helpfile_t4_to_t5')
        # django 1.4 doesn't have auth_message table
        if 'auth_message' in tables_list:
            tables_list.remove('auth_message')

        related_tables = {}
        # get a list of related tables for each table
        for table in tables_list:
            related_tables[table] = [field.rel.to._meta.db_table \
                        for field in models_d[table]._meta.fields \
                        if isinstance(field, (ForeignKey, OneToOneField))]

        sorted_list = []
        for table in tables_list:
            if not related_tables[table]:
                sorted_list.append(table)

        n = 100
        # avoid getting into the infinite loop - just in case
        #while related_tables:
        while n > 1:
            # remove tables from related_tables if already in the sorted_list
            for key in related_tables.keys():
                for rel_table in related_tables[key]:
                    if rel_table in sorted_list:
                        related_tables[key].remove(rel_table)
                if not related_tables[key]:
                    del related_tables[key]

            # add to the sorted_list if there is no
            # related_tables  for this table
            for table in tables_list:
                # if the related_tables is gone
                if table not in sorted_list and (
                       table not in related_tables.keys()):
                    sorted_list.append(table)

            # continue until all the related tables are gone
            if not related_tables:
                break

            n = n - 1

        if related_tables:
            print "ERROR: Sorting not completed."

        # copy the list to your conf.yml file
        print '-', '\n- '.join(sorted_list)



from datetime import date, timedelta
from django.utils.datastructures import SortedDict

def month_days(year, month):
    "Returns iterator for days in selected month"
    day = date(year, month, 1)
    while day.month == month:
        yield day
        day += timedelta(days=1) 

def request_month_range(request):
    "returns month start date and end date"
    now = date.today()
    year = int(request.GET.get('year') or str(now.year))
    month = int(request.GET.get('month') or str(now.month))
    days = list(month_days(year, month)) 
    return days[0], days[-1]


def day_bars(data, year, month, height, color_func=None):
    "Returns bars prepared for event-summary chart"
    
    def _sum_counts(items):
        for values in items.values():
            yield sum([i['count'] for i in values])
    
    if color_func:
        color_func(data)
    result = SortedDict([(d, []) for d in month_days(year, month)])
    for item in data:
        result[item['day']].append(item)
    try:
        kH = height*1.0 / max(_sum_counts(result))
    except Exception:
        kH = 1.0
    for values in result.values():
        for item in values:
            item['height'] = int(item['count']*kH)
    return result



from django import forms


class PayPalPaymentForm(forms.Form):
    # the _ext-enter is no longer required.
    cmd = forms.CharField(max_length=10,
                            widget=forms.HiddenInput,
                            initial='_ext-enter')
    redirect_cmd = forms.CharField(max_length=10,
                            widget=forms.HiddenInput,
                            initial='_xclick')
    business = forms.CharField(max_length=50, widget=forms.HiddenInput)
    image_url = forms.CharField(max_length=100, widget=forms.HiddenInput)
    display = forms.CharField(max_length=1,
                              widget=forms.HiddenInput,
                              initial='0')
    # do not prompt buyers to include a note with their payments.
    no_note = forms.CharField(max_length=1,
                              widget=forms.HiddenInput,
                              initial='1')
    # do not prompt buyers for a shipping address.
    no_shipping = forms.CharField(max_length=1,
                              widget=forms.HiddenInput,
                              initial='1')
    # link to thankyou page. add it later because we cannot add it here.
#    return = forms.CharField(max_length=100,
#                            widget=forms.HiddenInput)
    # form method used by thankyou page (2=POST)
    rm = forms.CharField(max_length=1,
                              widget=forms.HiddenInput,
                              initial='2')
    # indicates the use of third-party shopping cart
    upload = forms.CharField(max_length=1,
                              widget=forms.HiddenInput,
                              initial='0')
    paymentaction = forms.CharField(max_length=10,
                              widget=forms.HiddenInput,
                              initial='sale')
    # link to ipn
    notify_url = forms.CharField(max_length=100,
                                 widget=forms.HiddenInput)
    amount = forms.DecimalField(max_digits=15,
                                decimal_places=2,
                                widget=forms.HiddenInput)
    currency_code = forms.CharField(max_length=25,
                                    widget=forms.HiddenInput)
    charset = forms.CharField(max_length=10,
                              widget=forms.HiddenInput,
                              initial='utf-8')
    # pass through variable
    invoice = forms.IntegerField(widget=forms.HiddenInput)
    item_name = forms.CharField(max_length=1600, widget=forms.HiddenInput)

    first_name = forms.CharField(max_length=100, widget=forms.HiddenInput)
    last_name = forms.CharField(max_length=100, widget=forms.HiddenInput)
    email = forms.CharField(max_length=255, widget=forms.HiddenInput)
    address1 = forms.CharField(max_length=100, widget=forms.HiddenInput)
    address2 = forms.CharField(max_length=100, widget=forms.HiddenInput)
    city = forms.CharField(max_length=40, widget=forms.HiddenInput)
    state = forms.CharField(max_length=40, widget=forms.HiddenInput)
    zip = forms.CharField(max_length=7, widget=forms.HiddenInput)
    country = forms.CharField(max_length=20, widget=forms.HiddenInput)
    night_phone_a = forms.CharField(max_length=25, widget=forms.HiddenInput)



from django.conf import settings

from tendenci.core.registry import site
from tendenci.core.registry.base import CoreRegistry, lazy_reverse
from tendenci.addons.locations.models import Location


class LocationRegistry(CoreRegistry):
    version = '1.0'
    author = 'Schipul - The Web Marketing Company'
    author_email = 'programmers@schipul.com'
    description = 'A list of locations associated with your organization' \
        'Includes a search that sort by nearest location.'
    icon = '%simages/icons/locations-color-64x64.png' % settings.STATIC_URL

    event_logs = {
        'location':{
            'base':('830000', '669933'),
            'add':('831000', '669966'),
            'edit':('832000', '66CC66'),
            'delete':('833000', '66CC00'),
            'search':('834000', '66CC33'),
            'view':('835000', '66CC99'),
        }
    }

    url = {
        'add': lazy_reverse('location.add'),
        'search': lazy_reverse('location.search'),
        'list': lazy_reverse('locations'),
    }

site.register(Location, LocationRegistry)



from django.conf import settings
from tendenci.core.registry import site
from tendenci.core.registry.base import CoreRegistry, lazy_reverse
from tendenci.addons.articles.models import Article


class ArticleRegistry(CoreRegistry):
    version = '1.0'
    author = 'Schipul - The Web Marketing Company'
    author_email = 'programmers@schipul.com'
    description = 'Create articles to display basic content throughout the site'
    icon = '%simages/icons/articles-color-64x64.png' % settings.STATIC_URL

    event_logs = {
        'article': {
            'base': ('430000', 'CC9966'),
            'add': ('431000', 'CC9966'),
            'edit': ('432000', 'CCCC66'),
            'delete': ('433000', 'CCCC00'),
            'search': ('434000', 'CCCC33'),
            'view': ('435000', 'CCCC99'),
            'print_view': ('435001', 'FFCC99'),
        }
    }

    url = {
        'add': lazy_reverse('article.add'),
        'search': lazy_reverse('articles'),
        'list': lazy_reverse('articles'),
    }

site.register(Article, ArticleRegistry)



from tendenci.core.perms.managers import TendenciBaseManager


class ProfileManager(TendenciBaseManager):

    def create_profile(self, user):
        return self.create(user=user,
                           creator_id=user.id,
                           creator_username=user.username,
                           owner_id=user.id,
                           owner_username=user.username)

    def search(self, query=None, *args, **kwargs):
        """
        Uses haystack to user.
        Returns a SearchQuerySet.
        Filter out users if they have hide_in_search set to True.
        """
        sqs = super(ProfileManager, self).search(query=query, *args, **kwargs)
        if not kwargs.get('user').profile.is_superuser:
            sqs = sqs.filter(hide_in_search=False)
        return sqs

class ProfileActiveManager(TendenciBaseManager):
    def get_query_set(self):
        return super(ProfileActiveManager, self).get_query_set().filter(status=True, status_detail='active', user__is_active=True)



from datetime import datetime, timedelta
import operator
from haystack.query import SearchQuerySet

from django.db.models import Manager
from django.db.models import Q
from django.contrib.auth.models import User

from tendenci.core.perms.managers import TendenciBaseManager


class EventManager(TendenciBaseManager):
    def search(self, query=None, *args, **kwargs):
        """
        Uses haystack to query events.
        Returns a SearchQuerySet
        """
        sqs = super(EventManager, self).search(query=query, *args, **kwargs)
        
        start_dt, end_dt = kwargs.get('date_range', (None, None))
        
        if start_dt and end_dt:
            sqs = sqs.filter(start_dt__lte=start_dt, end_dt__gte=end_dt)
        else:
            # old behavior
            if start_dt:
                # this does not take into account events that are still active
                # this only takes into account events starting after the given date.
                sqs = sqs.filter(start_dt__gte=start_dt)
            elif end_dt:
                # this does not take into account events that are active before the given date.
                # this only takes into account events that will end before the given date.
                sqs = sqs.filter(end_dt__lte=end_dt)
        
        # sorting must be done outside this function
        # for some reason a 2nd call to order_by fails to sort again
        # if not query:
        # sort based on start_dt by default
        #    sqs = sqs.order_by('start_dt')
        #print sqs #will force the search query to be evaluated
        return sqs

    def search_filter(self, filters=None, *args, **kwargs):
        sqs = SearchQuerySet()
        user = kwargs.get('user', None)
        groups = []
        if user and user.is_authenticated():
            groups = [g.pk for g in user.group_set.all()]
        admin = user.profile.is_superuser
        
        # permission filters
        if user:
            if not user.profile.is_superuser:
                if not user.is_anonymous():
                    # (status+status_detail+(anon OR user)) OR (who_can_view__exact)
                    anon_query = Q(allow_anonymous_view=True)
                    user_query = Q(allow_user_view=True)
                    sec1_query = Q(status=True, status_detail='active')
                    user_perm_q = Q(users_can_view__in=[user.pk])
                    group_perm_q = Q(groups_can_view__in=groups)
                    
                    query = reduce(operator.or_, [anon_query, user_query])
                    query = reduce(operator.and_, [sec1_query, query])
                    query = reduce(operator.or_, [query, user_perm_q, group_perm_q])
                else:
                    sqs = sqs.filter(allow_anonymous_view=True)
        else:
            sqs = sqs.filter(allow_anonymous_view=True)
        
        # custom filters
        for filter in filters:
            sqs = sqs.filter(content='"%s"' % filter)
        
        return sqs.models(self.model)

class EventTypeManager(Manager):
    def search(self, query=None, *args, **kwargs):
        """
            Uses haystack to query events.
            Returns a SearchQuerySet
        """
        sqs = SearchQuerySet()
        user = kwargs.get('user', None)
        
        # check to see if there is impersonation
        if hasattr(user, 'impersonated_user'):
            if isinstance(user.impersonated_user, User):
                user = user.impersonated_user
        
        if query:
            sqs = sqs.auto_query(sqs.query.clean(query))
        
        return sqs.models(self.model)


class RegistrantManager(TendenciBaseManager):
    def search(self, query=None, *args, **kwargs):
        """
        Uses haystack to query events.
        Returns a SearchQuerySet
        """
        #sqs = SearchQuerySet()
        sqs = kwargs.pop('sqs', SearchQuerySet())
        event = kwargs.get('event')
        
        if event:
            sqs = sqs.filter(event_pk=event.pk)
        
        # let the parent search know that we have started a SQS
        kwargs.update({'sqs': sqs})
        
        sqs = super(RegistrantManager, self).search(
            query=query, *args, **kwargs)
        
        return sqs



import os
from django.conf import settings
from tendenci.core.site_settings.utils import get_setting
from tendenci.core.theme.middleware import get_current_request


def get_theme():
    request = get_current_request()
    if request:
        theme = request.session.get('theme', get_setting('module', 'theme_editor', 'theme'))
    else:
        theme = get_setting('module', 'theme_editor', 'theme')
    return theme


def get_theme_root(theme=None):
    if theme is None:  # default theme
        theme = get_theme()
    if settings.USE_S3_THEME:
        theme_root = theme
    else:
        # local theme root
        theme_root = (os.path.join(settings.ORIGINAL_THEMES_DIR, theme)).replace('\\', '/')
    return theme_root


def get_theme_template(template_name, theme=None):
    """Returns a relative path for the theme template.
    This is used primarily as an input for loader's get_template
    """
    if theme is None:  # default theme
        theme = get_theme()
    theme_template = template_name
    return theme_template


def theme_options():
    """
    Returns a string of the available themes in THEMES_DIR
    """
    options = ''
    themes = sorted(theme_choices())
    themes.reverse()
    for theme in themes:
        options = '%s, %s' % (theme, options)
    return options[:-2]


def theme_choices():
    """
    Returns a list of available themes in ORIGINAL_THEMES_DIR
    """
    themes_dir = settings.ORIGINAL_THEMES_DIR

    for theme in os.listdir(themes_dir):
        if os.path.isdir(os.path.join(themes_dir, theme)):
            yield theme



import re
import urllib2
from hashlib import md5

from tagging.templatetags.tagging_tags import TagsForObjectNode
from tagging.models import Tag
from BeautifulSoup import BeautifulStoneSoup

from django.utils.safestring import mark_safe
from django.template import Library, Node, Variable, TemplateSyntaxError
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.core.cache import cache

from tendenci.core.base.template_tags import parse_tag_kwargs
from tendenci.core.base.utils import url_exists
from tendenci.apps.profiles.models import Profile

from tendenci.core.files.cache import FILE_IMAGE_PRE_KEY
from tendenci.core.files.utils import generate_image_cache_key

register = Library()


@register.inclusion_tag("base/fb_like_iframe.html")
def fb_like_button_iframe(url, show_faces='false', width=400, height=30):
    from tendenci.core.site_settings.utils import get_setting
    site_url = get_setting('site', 'global', 'siteurl')
    url = '%s%s' % (site_url,url)
    if show_faces.lower() == 'true':
        show_faces = 'true'
    else:
        show_faces = 'false'
    try:
        width = int(width) 
    except:
        width = 400
    try:
        height = int(height) 
    except:
        height = 400
        
    return {'url': url, 
            'width': width,
            'height': height,
            'show_faces':show_faces}

class FanCountNode(Node):
    def __init__(self, service, service_id):
        self.service = service
        self.service_id = service_id
    
    def render(self, context):
        fancount = ''
        fb_api_url = 'http://api.facebook.com/restserver.php'
        tw_api_url = 'http://api.twitter.com'
        
        cache_key = ''
        cache_time = 1800
        
        if self.service == "facebook":
            query = '%s?method=facebook.fql.query&query=SELECT%%20fan_count%%20FROM%%20page%%20WHERE%%20page_id=%s'
            xml_path = query % (fb_api_url, self.service_id)
            cache_key = md5(xml_path).hexdigest()
            fancount = cache.get(cache_key)
            if not fancount:
                try:
                    xml = urllib2.urlopen(xml_path)
                    content = xml.read()
                    soup = BeautifulStoneSoup(content)
                    nodes = soup.findAll('page')
                    for node in nodes:
                        fancount = node.fan_count.string
                    cache.set(cache_key, fancount, cache_time)
                except:
                    pass

        if self.service == "twitter":
            query = "%s/1/users/show/%s.xml"
            xml_path = query % (tw_api_url, self.service_id)
            cache_key = md5(xml_path).hexdigest()
            fancount = cache.get(cache_key)
            if not fancount:
                try:
                    xml = urllib2.urlopen(xml_path)
                    content = xml.read()
                    soup = BeautifulStoneSoup(content)
                    nodes = soup.findAll('user')
                    for node in nodes:
                        fancount = node.followers_count.string
                    cache.set(cache_key, fancount, cache_time)
                except:
                    pass

        return fancount
    
@register.tag
def fan_count(parser, token):
    """
    Pull a fan count for a social media site:
    
    Usage::

        {% fan_count facebook 12345 %}

        or

        {% fan_count twitter username %}
    
    """
    bits = token.contents.split()
    if len(bits) < 3:
        raise TemplateSyntaxError("'%s' tag takes two arguments" % bits[0])
    service = bits[1]
    service_id = bits[2]
    return FanCountNode(service, service_id)

def callMethod(obj, methodName):
    """
        callMethod and args are used so that we can call a method with parameters in template
        Example:
            if you want to call: user_this.allow_view_by(user_current)
            in template: {{ user_this.get_profile|args:user_current|call:'allow_edit_by' }}
    """
    method = getattr(obj, methodName)
 
    if obj.__dict__.has_key("__callArg"):
        ret = method(*obj.__callArg)
        del obj.__callArg
        return ret
    return method()
 
def args(obj, arg):
    if not obj.__dict__.has_key("__callArg"):
        obj.__callArg = []
     
    obj.__callArg += [arg]
    return obj
 
register.filter("call", callMethod)
register.filter("args", args)


# so we can create new variables in template
# {% assign [name] [value] %}
class AssignNode(Node):
    def __init__(self, name, value):
        self.name = name
        self.value = value
        
    def render(self, context):
        context[self.name] = self.value.resolve(context, True)
        return ''

def do_assign(parser, token):
    """
    Assign an expression to a variable in the current context.
    
    Usage::
        {% assign [name] [value] %}
    Example::
        {% assign list entry.get_related %}
        
    """
    bits = token.contents.split()
    if len(bits) != 3:
        raise TemplateSyntaxError("'%s' tag takes two arguments" % bits[0])
    value = parser.compile_filter(bits[2])
    return AssignNode(bits[1], value)

register.tag('assign', do_assign)


class GetProfileNode(Node):
    def __init__(self, obj, var_name):
        self.user_obj = obj
        self.var_name = var_name
        
    def resolve(self, var, context):
        """Resolves a variable out of context if it's not in quotes"""
        if var[0] in ('"', "'") and var[-1] == var[0]:
            return var[1:-1]
        else:
            return Variable(var).resolve(context)
        
    def render(self, context):
        user_obj = self.resolve(self.user_obj, context)
        var_name = self.resolve(self.var_name, context)
        if isinstance(user_obj, User):
            try:
                profile = user_obj.get_profile()
            except Profile.DoesNotExist:
                profile = Profile.objects.create_profile(user=user_obj)
            context[var_name] = profile
        return ""

def get_or_create_profile_for(parser, token):
    """
    Get or create a user profile if not exists
    
    Example::
        {% get_or_create_profile user_this %}
        
    """
    bits = token.contents.split()
    if len(bits) < 2:
        raise TemplateSyntaxError("'%s' tag takes at least two arguments" % bits[0])
    
    args = {
        'obj': bits[1],
        'var_name': next_bit_for(bits, 'as'),
    }
    return GetProfileNode(**args)

def next_bit_for(bits, key, if_none=None, step=1):
    try:
        return bits[bits.index(key)+step]
    except ValueError:
        return if_none

register.tag('get_or_create_profile_for', get_or_create_profile_for)

class ResetNode(Node):
    def __init__(self, variable, **kwargs):
        # set the context var
        self.variable = Variable(variable)
            
        # get the context vars
        for k, v in kwargs.items():
            if k == 'context':
                self.context = v
            
    def render(self, context):
        variable = self.variable.resolve(context)
        context[self.context] = variable 
        return ''

@register.tag      
def reset(parser, token):
    """
    Reset a context variable to another one
    
    Usage::
        {% reset var as var1 %} 
    """
    bits = token.split_contents()

    try:
        variable = bits[1]
    except:
        raise TemplateSyntaxError, "'%s' requires at least three arguments." % bits[0]
    
    if bits[1] == 'as':
        raise TemplateSyntaxError, "'%s' first argument must be a context var." % bits[0]
    
    # get the user
    try:
        variable = bits[bits.index('as')-1]
        context = bits[bits.index('as')+1]
    except:
        variable = None
        context = None
    
    if not variable and not context:
        raise TemplateSyntaxError, "'%s' missing arguments. Syntax {% reset var1 as var2 %}" % bits[0]
    
    return ResetNode(variable, context=context)

class ImagePreviewNode(Node):
    def __init__(self, instance, size, **kwargs):
        # set the context var
        self.instance = Variable(instance)    
        self.size = size
        self.context = None
        
        # get the context vars
        for k, v in kwargs.items():
            if k == 'context':
                self.context = v
            
    def render(self, context):
        instance = self.instance.resolve(context)
        
        app_label = instance._meta.app_label
        model = instance._meta.object_name.lower()
        
        url = reverse('image_preview', args=(app_label, model, instance.id, self.size))
        if not url_exists(url):
            url = None
              
        if self.context:
            context[self.context] = url 
        else:
            context['image_preview'] = url
        
        return ''

@register.tag      
def image_preview(parser, token):
    """
        Gets the url for an image preview base
        on model and model_id
    """
    bits = token.split_contents()
    
    try:
        instance = bits[1]
    except:
        raise TemplateSyntaxError, "'%s' requires at least 2 arguments" % bits[0]
        
    try:
        size = bits[2]
    except:
        raise TemplateSyntaxError, "'%s' requires at least 2 arguments" % bits[0]        
         
    try:
        context = bits[4]
    except:
        context = "image_preview"
        
    return ImagePreviewNode(instance, size, context=context)


class RssParserNode(Node):
    def __init__(self, context_var, url, *args, **kwargs):
        self.url = url
        self.context_var = context_var
        self.kwargs = kwargs

    def render(self, context):
        import feedparser

        cache_timeout = 300

        if 'cache' in self.kwargs:
            try:
                cache_timeout = Variable(self.kwargs['cache'])
                cache_timeout = cache_timeout.resolve(context)
            except:
                cache_timeout = self.kwargs['cache']

        cache_key = md5(self.url).hexdigest()
        url_content = cache.get(cache_key)

        if url_content is None:
            url_content = feedparser.parse(self.url)
            # We are going to try to pop out the errors in the
            # feed because they raise an exception that can't be
            # pickled when we try to cache the content.
            if 'bozo_exception' in url_content:
                url_content['bozo_exception'] = ''
            cache.set(cache_key, url_content, int(cache_timeout))

        context[self.context_var] = url_content

        return ''


@register.tag(name="get_rss")
def get_rss(parser, token):
    """
    Take an RSS feed so you can iterate through the entries.

    Usage::

        {% get_rss [rss_feed_url] as [variable] cache=600 %}

    Options include:

        ``cache``
           The length of time to cache the feed in seconds. **Default: 300**

    Example::

        {% get_rss "http://www.freesound.org/blog/?feed=rss2" as rss %}
        {% for entry in rss.entries %}
            <h1>{{entry.title}}</h1>
            <p>
                {{entry.summary|safe}}
            </p>
            <p>
                <a href="{{entry.link}}">read more...</a>
            </p>
        {% endfor %}
    """
    args, kwargs = [], {}
    bits = token.split_contents()
    url_string = bits[1]
    context_var = bits[3]

    if len(bits) < 4:
        message = "'%s' tag requires more than 3" % bits[0]
        raise TemplateSyntaxError(message)

    if bits[2] != "as":
        message = "'%s' third argument must be 'as" % bits[0]
        raise TemplateSyntaxError(message)

    kwargs = parse_tag_kwargs(bits)

    if url_string[0] == url_string[-1] and url_string[0] in ('"', "'"):
        url = url_string[1:-1]
    else:
        url = url_string

    return RssParserNode(context_var, url, *args, **kwargs)


class Md5Hash(Node):
    
    def __init__(self, *args, **kwargs):
        self.bits = [Variable(bit) for bit in kwargs.get("bits", [])[1:]]

    def render(self, context):
        from hashlib import md5
        bits = [str(b.resolve(context)) for b in self.bits]
        return md5(".".join(bits)).hexdigest()

@register.tag
def md5_hash(parser, token):
    """
    Example::
        {% md5_hash obj.pk obj.email %}
    """
    args, kwargs = [], {}
    kwargs["bits"] = token.split_contents()

    if len(kwargs["bits"]) < 2:
        message = "'%s' tag requires more than 2" % kwargs["bits"][0]
        raise TemplateSyntaxError(message)

    return Md5Hash(*args, **kwargs)

class NoWhiteSpaceNode(Node):
    def __init__(self, nodelist):
        self.nodelist = nodelist

    def render(self, context):
        text = self.nodelist.render(context).strip()
        return re.sub('\s{2,}', ' ', text)

@register.tag
def nowhitespace(parser, token):
    """
    Replaces two or more spaces between regular text
    (*NOT* HTML) down to on space
    """
    nodelist = parser.parse(('endnowhitespace',))
    parser.delete_first_token()
    return NoWhiteSpaceNode(nodelist)


class PhotoImageURL(Node):
    def __init__(self, photo, *args, **kwargs):
        self.size = kwargs.get("size", "100x100")
        self.crop = kwargs.get("crop", False)
        self.constrain = kwargs.get("constrain", False)
        self.quality = kwargs.get("quality", 90)
        self.photo = Variable(photo)

    def render(self, context):
        photo = self.photo.resolve(context)

        # We can't crop and constrain, so we need
        # to pick one if both are passed
        if self.crop and self.constrain:
            self.constrain = False

        # return empty unicode string
        if not photo.pk:
            return unicode()

        cache_key = generate_image_cache_key(file=str(photo.pk), size=self.size, pre_key="photo", crop=self.crop, unique_key=str(photo.pk), quality=self.quality, constrain=self.constrain)
        cached_image_url = cache.get(cache_key)
        if cached_image_url:
            return cached_image_url

        args = [photo.pk, self.size]
        if self.crop:
            args.append("crop")
        if self.constrain:
            args.append("constrain")
        if self.quality:
            args.append(self.quality)
        url = reverse('photo.size', args=args)
        return url


@register.tag
def photo_image_url(parser, token):
    """
    Creates a url for a photo that can be resized, cropped, and have quality reduced.
    Used specifically for photos.

    Example::

        {% list_photos as photos user=user limit=3 %}
        {% for photo in photos %}
            <img src="{% photo_image_url photo size=100x100 crop=True constrain=True %}" />
        {% endfor %}
    """
    args, kwargs = [], {}
    bits = token.split_contents()
    photo = bits[1]

    for bit in bits:
        if "size=" in bit:
            kwargs["size"] = bit.split("=")[1]
        if "crop=" in bit:
            kwargs["crop"] = bool(bit.split("=")[1])
        if "constrain=" in bit:
            kwargs["constrain"] = bool(bit.split("=")[1])
        if "quality=" in bit:
            kwargs["quality"] = bit.split("=")[1]

    if len(bits) < 1:
        message = "'%s' tag requires more than 1 argument" % bits[0]
        raise TemplateSyntaxError(message)

    return PhotoImageURL(photo, *args, **kwargs)


class ImageURL(Node):
    def __init__(self, file, *args, **kwargs):
        self.size = kwargs.get("size", None)
        self.crop = kwargs.get("crop", False)
        self.quality = kwargs.get("quality", None)
        self.constrain = kwargs.get("constrain", None)
        self.file = Variable(file)

    def render(self, context):
        file = self.file.resolve(context)

        if file and file.pk:

            cache_key = generate_image_cache_key(file=str(file.id), size=self.size, pre_key=FILE_IMAGE_PRE_KEY, crop=self.crop, unique_key=str(file.id), quality=self.quality, constrain=self.constrain)
            cached_image_url = cache.get(cache_key)
            if cached_image_url:
                return cached_image_url

            args = [file.pk]
            if self.size:
                args.append(self.size)
            if self.crop:
                args.append("crop")
            if self.quality:
                args.append(self.quality)
            url = reverse('file', args=args)
            return url
        # return empty unicode string
        return unicode('')


@register.tag
def image_url(parser, token):
    """
    Creates a url for a photo that can be resized, cropped, and have quality reduced.
    
    Usage::

        {% image_url file [options][size=100x100] [crop=True] [quality=90] %}

    Options include:
    
        ``size``
           The size in the format [width]x[height]. **Default: 100x100**
        ``crop``
           Whether or not to crop the image. **Default: False**
        ``quality``
           The quality of the rendered image. Use smaller for faster loading. **Default: 90**

    Example::

        <img src="{% image_url file size=150x100 crop=True quality=90 %}" />
    """
    args, kwargs = [], {}
    bits = token.split_contents()
    file = bits[1]

    for bit in bits:
        if "size=" in bit:
            kwargs["size"] = bit.split("=")[1]
        if "crop=" in bit:
            kwargs["crop"] = bool(bit.split("=")[1])
        if "quality=" in bit:
            kwargs["quality"] = bit.split("=")[1]

    if len(bits) < 1:
        message = "'%s' tag requires more than 1 argument" % bits[0]
        raise TemplateSyntaxError(message)

    return ImageURL(file, *args, **kwargs)
    
class NonHashedTagsNode(TagsForObjectNode):
    def render(self, context):
        context[self.context_var] = \
            Tag.objects.get_for_object(self.obj.resolve(context)).exclude(
            name__contains='#')
        return ''
        
class HashedTagsNode(TagsForObjectNode):
    def render(self, context):
        context[self.context_var] = \
            Tag.objects.get_for_object(self.obj.resolve(context)).filter(
            name__contains='#')
        return ''
        
def do_non_hash_tags_for_object(parser, token):
    """
    Retrieves a list of ``Tag`` objects that DO NOT start with '#'
    associated with an object and stores them in a context variable.
    
    Usage::
    
       {% tags_strip_hash [object] as [varname] %}
    
    Example::
    
        {% tags_strip_hash foo_object as tag_list %}
    """
    bits = token.contents.split()
    if len(bits) != 4:
        raise TemplateSyntaxError(_('%s tag requires exactly three arguments') % bits[0])
    if bits[2] != 'as':
        raise TemplateSyntaxError(_("second argument to %s tag must be 'as'") % bits[0])
    return NonHashedTagsNode(bits[1], bits[3])
    
def do_hash_tags_for_object(parser, token):
    """
    Retrieves a list of ``Tag`` objects that START with '#'
    associated with an object and stores them in a context variable.
    
    Usage::
    
       {% tags_hash_tags[object] as [varname] %}
    
    Example::
    
        {% tags_hash_tags foo_object as tag_list %}
    """
    bits = token.contents.split()
    if len(bits) != 4:
        raise TemplateSyntaxError(_('%s tag requires exactly three arguments') % bits[0])
    if bits[2] != 'as':
        raise TemplateSyntaxError(_("second argument to %s tag must be 'as'") % bits[0])
    return HashedTagsNode(bits[1], bits[3])
    
register.tag('tags_strip_hash', do_non_hash_tags_for_object)
register.tag('tags_hash_tags', do_hash_tags_for_object)

@register.inclusion_tag("base/meta_creator_owner.html")
def meta_creator_owner(obj):
    return {'obj': obj}

@register.inclusion_tag("base/stock_image_url.html", takes_context=True)
def stock_image_url(context, size):
    context.update({'size': size})
    return context


@register.simple_tag
def all_tags_list():
    """
    Creates a text list of tags for use in a dropdown
    """
    tags = Tag.objects.all()
    tag_list = ",".join(['"%s"' % t.name for t in tags])
    return mark_safe(tag_list)



from django.conf import settings

from tendenci.core.registry import site
from tendenci.core.registry.base import PeopleRegistry, lazy_reverse
from tendenci.apps.profiles.models import Profile


class ProfileRegistry(PeopleRegistry):
    version = '1.0'
    author = 'Schipul - The Web Marketing Company'
    author_email = 'programmers@schipul.com'
    description = 'User Profiles.'
    icon = '%simages/icons/users-color-64x64.png' % settings.STATIC_URL
    
    event_logs = {
        'profile':{
            'base':('120000','3300FF'),
            'add':('121000','3300FF'),
            'edit':('122000', '3333FF'),
            'delete':('123000', '3366FF'),
            'search':('124000', '3366FF'),
            'view':('125000', '3399FF'),
        },
    }
    
    url = {
        'search': lazy_reverse('profile.search'),
        'add': lazy_reverse('profile.add'),
        'settings': lazy_reverse('settings.index', args=['module','users']),
    }

site.register(Profile, ProfileRegistry)



from django.template import RequestContext
from django.shortcuts import render_to_response

from tendenci.core.perms.decorators import superuser_required
from tendenci.core.handler404.models import Report404
from tendenci.core.event_logs.models import EventLog


@superuser_required
def reports_404(request, template_name='reports/404_report.html'):
    entry_list = Report404.objects.all().order_by('-count')

    EventLog.objects.log()

    return render_to_response(template_name, {
        'entry_list': entry_list,
    }, context_instance=RequestContext(request))



import operator
from haystack.query import SearchQuerySet

from django.db.models import Manager
from django.db.models import Q
from django.contrib.auth.models import User

from tendenci.core.perms.managers import TendenciBaseManager

class GroupManager(TendenciBaseManager):
    pass

class OldGroupManager(Manager):
    def search(self, query=None, *args, **kwargs):
        """
            Uses haystack to query groups. 
            Returns a SearchQuerySet
        """
        sqs = SearchQuerySet()
        user = kwargs.get('user', None)

        # check to see if there is impersonation
        if hasattr(user,'impersonated_user'):
            if isinstance(user.impersonated_user, User):
                user = user.impersonated_user
                
        is_a_superuser = user.profile.is_superuser
            
        if query:
            sqs = sqs.auto_query(sqs.query.clean(query)) 
            if user:
                if not is_a_superuser:
                    if not user.is_anonymous():
                    # if b/w superuser and anon

                        # (status+status_detail+(anon OR user)) OR (who_can_view__exact)
                        anon_query = Q(**{'allow_anonymous_view':True,})
                        user_query = Q(**{'allow_user_view':True,})
                        sec1_query = Q(**{
                            'show_as_option':1,
                            'status':1,
                            'status_detail':'active',
                        })
                        sec2_query = Q(**{
                            'who_can_view__exact':user.username
                        })
                        query = reduce(operator.or_, [anon_query, user_query])
                        query = reduce(operator.and_, [sec1_query, query])
                        query = reduce(operator.or_, [query, sec2_query])

                        sqs = sqs.filter(query)
                    else: # anonymous
                        query = Q(**{
                            'allow_anonymous_view': True,
                            'show_as_option':1,
                            'status':1,
                            'status_detail':'active',
                        })
                        sqs = sqs.filter(query)
            else: # anonymous
                query = Q(**{
                    'allow_anonymous_view': True,
                    'show_as_option':1,
                    'status':1,
                    'status_detail':'active',
                })
                sqs = sqs.filter(query)
        else:
            if user:
                if is_a_superuser:
                    # this is no-op. the .all().exclude(type='membership').models(Group) wouldn't work
                    #sqs = sqs.all()
                    sqs = sqs.filter(status=True)
                else:
                    if not user.is_anonymous():
                        # (status+status_detail+anon OR who_can_view__exact)
                        anon_query = Q(**{'allow_anonymous_view':True,})
                        user_query = Q(**{'allow_user_view':True,})
                        sec1_query = Q(**{
                            'show_as_option':1,
                            'status':1,
                            'status_detail':'active',
                        })
                        sec2_query = Q(**{
                            'who_can_view__exact': user.username
                        })
                        query = reduce(operator.or_, [anon_query, user_query])
                        query = reduce(operator.and_, [sec1_query, query])
                        query = reduce(operator.or_, [query, sec2_query])
                        sqs = sqs.filter(query)
                    else: # anonymous
                        query = Q(**{
                            'allow_anonymous_view': True,
                            'show_as_option':1,
                            'status':1,
                            'status_detail':'active',
                        })
                        sqs = sqs.filter(query)   
            else: # anonymous
                query = Q(**{
                    'allow_anonymous_view': True,
                    'show_as_option':1,
                    'status':1,
                    'status_detail':'active',
                })
                sqs = sqs.filter(query)
        
        sqs = sqs.order_by('-create_dt')
        
        return sqs.models(self.model)
    



from django import template

from tendenci.core.site_settings.utils import get_setting
from django.utils.safestring import mark_safe

register = template.Library()

GLOBAL_SITE_URL = get_setting('site', 'global', 'siteurl')

def relative_to_absolute_urls(value):
    """
    Converts all relative urls to absolute urls.
    Automatically marks the strings as safe.
    e.g.
    {{ event.description|relative_to_absolute_urls }}
    """
    value = value.replace('src="/', 'src="%s/' % GLOBAL_SITE_URL)
    value = value.replace("src='/", "src='%s/" % GLOBAL_SITE_URL)
    value = value.replace('href="/', 'href="%s/' % GLOBAL_SITE_URL)
    value = value.replace("href='/", "<href='%s/" % GLOBAL_SITE_URL)
    return mark_safe(value)
    
register.filter('relative_to_absolute_urls', relative_to_absolute_urls)
